from bs4 import BeautifulSoup

from automon.log import Logging

log = Logging(name='BeautifulSoupClient', level=Logging.DEBUG)


class BeautifulSoupClient(object):

    def __init__(self, bs: BeautifulSoup = None):
        self.bs = bs

    def read_markup(self, markup: str, features: str = 'lxml'):
        """read markup with beautifulsoup"""
        try:
            self.bs = BeautifulSoup(
                markup=markup or self.markup,
                features=features
            )
            log.info(f'read_markup success ({len(markup)} B)')
        except Exception as e:
            log.error(f'read_markup failed ({len(markup)} B): {e}')

        return self



import json

from .datatypes import ServiceNow


class ServiceNowTicket(ServiceNow):
    active = None
    activity_due = None
    additional_assignee_list = None
    approval = None
    approval_history = None
    approval_set = None
    assigned_to = None
    assignment_group = None
    business_duration = None
    business_service = None
    business_stc = None
    calendar_duration = None
    calendar_stc = None
    caller_id = None
    category = None
    caused_by = None
    child_incidents = None
    close_code = None
    close_notes = None
    closed_at = None
    closed_by = None
    cmdb_ci = None
    comments = None
    comments_and_work_notes = None
    company = None
    contact_type = None
    correlation_display = None
    correlation_id = None
    delivery_plan = None
    delivery_task = None
    description = None
    due_date = None
    escalation = None
    expected_start = None
    follow_up = None
    group_list = None
    impact = None
    incident_state = None
    knowledge = None
    location = None
    made_sla = None
    notify = None
    number = None
    opened_at = None
    opened_by = None
    order = None
    parent = None
    parent_incident = None
    priority = None
    problem_id = None
    reassignment_count = None
    rejection_goto = None
    reopen_count = None
    resolved_at = None
    resolved_by = None
    rfc = None
    severity = None
    short_description = None
    sla_due = None
    state = None
    subcategory = None
    sys_class_name = None
    sys_created_by = None
    sys_created_on = None
    sys_domain = None
    sys_domain_path = None
    sys_id = None
    sys_mod_count = None
    sys_tags = None
    sys_updated_by = None
    sys_updated_on = None
    time_worked = None
    upon_approval = None
    upon_reject = None
    urgency = None
    user_input = None
    watch_list = None
    wf_activity = None
    work_end = None
    work_notes = None
    work_notes_list = None
    work_start = None

    @property
    def number(self):
        return self.sys_id

    def add_property(self, key, value):
        return self.__dict__.update({key: value})

    def to_api(self):
        d = self.to_dict()
        d['number'] = self.number
        return json.dumps(d)



import unittest

from automon.integrations.instagram.client_browser import InstagramBrowserClient


class InstagramClientTest(unittest.TestCase):
    c = InstagramBrowserClient(headless=True)
    c.browser.run()

    if c.is_running():
        c.browser.get(c.urls.login_page)

        # user
        login_user = c.browser.wait_for_xpath(c.xpaths.login_user)
        c.browser.action_click(login_user, 'user')
        c.browser.action_type(c.login)

        # password
        password = c.browser.wait_for_xpath(c.xpaths.login_pass)
        c.browser.action_click(password, 'password')

        c.browser.quit()


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.helpers import Run


class RunTest(unittest.TestCase):
    r = Run()

    def test_Run(self):
        self.assertTrue(Run)
        self.assertFalse(Run())

    def test_set_command(self):
        self.assertTrue(self.r.set_command('which'))
        self.assertFalse(self.r.set_command(''))

    def test_which(self):
        self.assertTrue(self.r.which('which'))
        self.assertFalse(self.r.which(''))

    def test_run_command(self):
        self.assertTrue(self.r.run_command('id'))
        self.assertFalse(self.r.run_command('false'))

    def test_run(self):
        self.assertTrue(self.r.run('id'))
        self.assertFalse(self.r.run('false'))


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.elasticsearch.client import ElasticsearchClient


class Elasticsearch(unittest.TestCase):

    def test_endpoints(self):
        c = ElasticsearchClient()

        if c.connected():
            self.assertTrue(c)
            self.assertTrue(c.get_indices())
            self.assertTrue(c.info())
            self.assertTrue(c.ping())


if __name__ == '__main__':
    unittest.main()



from automon.log import logger
from automon.helpers.osWrapper import environ
from automon.integrations.google.auth import GoogleAuthConfig

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class GoogleSheetsConfig(GoogleAuthConfig):
    """Google Sheets config"""

    def __init__(
            self,
            spreadsheetId: str = None,
    ):
        super().__init__()

        self.serviceName = 'sheets'
        self.scopes = [
            'https://www.googleapis.com/auth/drive',
            'https://www.googleapis.com/auth/drive.file',
            'https://www.googleapis.com/auth/drive.readonly',
            'https://www.googleapis.com/auth/spreadsheets',
            'https://www.googleapis.com/auth/spreadsheets.readonly',
        ]
        self.version = 'v4'

        self.spreadsheetId = spreadsheetId or environ('GOOGLE_SHEET_ID')

        log.info(f'{self}')



import neo4j
import logging

from neo4j import GraphDatabase
from queue import Queue

from automon.log import logger
from automon.integrations.neo4jWrapper.cypher import Cypher

from .config import Neo4jConfig
from .results import Results

logger.logging.getLogger('neo4j').setLevel(logging.ERROR)
log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class Neo4jClient:

    def __init__(self,
                 user: str = None,
                 password: str = None,
                 hosts: list or str = None,
                 config: Neo4jConfig = None) -> neo4j:
        """Neo4j client"""

        self._config = config or Neo4jConfig(
            user=user,
            password=password,
            hosts=hosts
        )

        self._client = self._client()

        if self._client:
            self._driver = self._client
            self._session = self._client.session()

        self._Cypher = Cypher()

        self.queue = Queue()
        self.cypher = ''
        self.results = None

    def __repr__(self):
        return f'{self.__dict__}'

    def _client(self):
        try:
            client = GraphDatabase.driver(
                uri=self._config.NEO4J_HOST,
                auth=(self._config.NEO4J_USER, self._config.NEO4J_PASSWORD))
            log.info(f'Connected to neo4j server: {self._config.NEO4J_HOST}')
            return client

        except Exception as e:
            log.error(f'Cannot connect to neo4j server: {self._config.NEO4J_HOST}, {e}')

        return False

    def _send(self, cypher: str or Cypher) -> GraphDatabase.driver:
        """Send cypher to server"""

        if not self.isConnected():
            return False

        if isinstance(cypher, Cypher):
            cypher = self._Cypher.consolidate()

        self.cypher = cypher

        return self.run()

    def build_cypher(self, query: str):
        """Build a cypher"""
        self.cypher += query

    def create(self, prop: str,
               value: str,
               label: str = None,
               node: str = None, **kwargs):
        """Create a node"""
        cypher = self._Cypher.create(prop=prop, value=value, node=node, label=label, **kwargs)
        self.cypher = cypher
        return self.run()

    def create_dict(self, prop: str,
                    value: str,
                    data: dict,
                    label: str = None,
                    node: str = None, **kwargs):
        """Create a node from dict"""
        cypher = self._Cypher.create_dict(prop=prop, value=value, label=label, node=node, data=data, **kwargs)
        self.cypher = cypher
        return self.run()

    def relationship(self,
                     A_node: str = 'A', A_label: str = None, A_prop: str = None, A_value: str = None,
                     B_node: str = 'B', B_label: str = None, B_prop: str = None, B_value: str = None,
                     WHERE: str = None, label: str = None, node: str = 'r', direction: str = '->'):
        """Create relationship between two existing nodes"""

        cypher = self._Cypher.relationship(
            A_node=A_node, A_label=A_label, A_prop=A_prop, A_value=A_value,
            B_node=B_node, B_label=B_label, B_prop=B_prop, B_value=B_value,
            WHERE=WHERE, label=label, node=node, direction=direction)

        self.cypher = cypher
        return self.run()

    def cypher_run(self, query: str):
        """Run a cypher query"""
        self.cypher = query
        return self.run()

    def delete_all(self):
        """Delete all nodes and relationships"""
        self.cypher = self._Cypher.delete_all()
        return self.run()

    def delete_match(self, prop: str, value: str, node: str = None):
        """Delete all matching nodes and its relationships"""
        cypher = self._Cypher.delete_node(prop=prop, value=value, node=node)
        self.cypher = cypher
        return self.run()

    def isConnected(self):
        """Check if client is connected to server"""
        if self._client:
            return True
        return False

    def match(self, prop: str, value: str, node: str = None):
        cypher = self._Cypher.match(prop=prop, value=value, node=node)
        self.cypher = cypher
        return self.run()

    def merge(self, prop: str = None, value: str = None, node: str = None, label: str = ''):
        """Merge nodes"""
        cypher = self._Cypher.merge(prop=prop, value=value, node=node, label=label)
        self.cypher = cypher
        return self.run()

    def merge_dict(self, prop: str, value: str, data: dict, label: str = None, node: str = None, **kwargs):
        """Merge nodes from dict"""

        # cypher_assertions = []
        # TODO: find a way to check if assertion has already been made
        # query = 'CREATE CONSTRAINT ON ( {node} {label} ) ASSERT {node}.raw IS UNIQUE'
        # query = query.format(node=node, label=label)
        # cypher_assertions.append(query)  # Pre cypher query

        cypher = self._Cypher.merge_dict(prop=prop, value=value, label=label, data=data, node=node, **kwargs)
        self.cypher = cypher
        return self.run()

    def merge_node(self, data: dict, label: str = None, **kwargs):
        """Merge a node"""

        # cypher_assertions = []
        # TODO: find a way to check if assertion has already been made
        # query = 'CREATE CONSTRAINT ON ( {node} {label} ) ASSERT {node}.raw IS UNIQUE'
        # query = query.format(node=node, label=label)
        # cypher_assertions.append(query)  # Pre cypher query

        cypher = self._Cypher.merge_dict(label=label, data=data)

        # log.debug(f'{final_cypher}')

        return self._send(cypher)

    def run(self, cypher=None) -> bool:
        """Send the cypher query to the server"""
        if not self.isConnected():
            return False

        cypher = self.cypher

        try:
            response = self._session.run(cypher)
            self.results = Results(response)

            log.info(f'cypher: {cypher}')
            log.debug(f'Results: {self.results}')

            return True
        except Exception as e:
            log.error(f"{e}")

        return False

    def search(self, prop: str, value: str, node: str = None):
        cypher = self._Cypher.match(prop=prop, value=value, node=node)
        cypher += self._Cypher.return_all()
        self.cypher = cypher
        return self.run()



import os

from automon.log import logger

log = logger.logging.getLogger(__name__)
log.setLevel(logger.ERROR)


class SlackConfig(object):

    def __init__(self, username=None,
                 webhook=None,
                 proxy=None,
                 token=None,
                 channel=None):
        """Slack config v2
        """

        self.username = username or os.getenv('SLACK_USER') or ''
        self.webhook = webhook or os.getenv('SLACK_WEBHOOK') or ''
        self.proxy = proxy or os.getenv('SLACK_PROXY') or ''
        self.token = token or os.getenv('SLACK_TOKEN') or ''

        self.channel = channel or ''

        self.SLACK_DEFAULT_CHANNEL = os.getenv('SLACK_DEFAULT_CHANNEL') or ''
        self.SLACK_INFO_CHANNEL = os.getenv('SLACK_INFO_CHANNEL') or ''
        self.SLACK_DEBUG_CHANNEL = os.getenv('SLACK_DEBUG_CHANNEL') or ''
        self.SLACK_ERROR_CHANNEL = os.getenv('SLACK_ERROR_CHANNEL') or ''
        self.SLACK_CRITICAL_CHANNEL = os.getenv('SLACK_CRITICAL_CHANNEL') or ''
        self.SLACK_WARN_CHANNEL = os.getenv('SLACK_WARN_CHANNEL') or ''
        self.SLACK_TEST_CHANNEL = os.getenv('SLACK_TEST_CHANNEL') or ''

        if not self.token:
            log.warning(f'missing SLACK_TOKEN')


class ConfigSlack:
    """Slack config v1
    """

    slack_name = os.getenv('SLACK_USER') or ''
    slack_webhook = os.getenv('SLACK_WEBHOOK') or ''
    slack_proxy = os.getenv('SLACK_PROXY') or ''
    slack_token = os.getenv('SLACK_TOKEN') or ''

    SLACK_DEFAULT_CHANNEL = os.getenv('SLACK_DEFAULT_CHANNEL') or ''
    SLACK_INFO_CHANNEL = os.getenv('SLACK_INFO_CHANNEL') or ''
    SLACK_DEBUG_CHANNEL = os.getenv('SLACK_DEBUG_CHANNEL') or ''
    SLACK_ERROR_CHANNEL = os.getenv('SLACK_ERROR_CHANNEL') or ''
    SLACK_CRITICAL_CHANNEL = os.getenv('SLACK_CRITICAL_CHANNEL') or ''
    SLACK_WARN_CHANNEL = os.getenv('SLACK_WARN_CHANNEL') or ''
    SLACK_TEST_CHANNEL = os.getenv('SLACK_TEST_CHANNEL') or ''

    if not slack_token:
        log.warning(f'missing SLACK_TOKEN')

    def __init__(self, slack_name: str = ''):
        self.slack_name = os.getenv('SLACK_USER') or slack_name or ''



from automon.log import logger

from .datatypes import AbstractDataType

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class Asset(AbstractDataType):
    def __init__(self, asset: dict = {}):
        self.__dict__.update(asset)



import unittest

from automon.integrations.google.auth import GoogleAuthConfig


class MyTestCase(unittest.TestCase):
    def test_something(self):
        test = GoogleAuthConfig()
        if test.Credentials:
            self.assertTrue(test.Credentials)


if __name__ == '__main__':
    unittest.main()



import json

from dateutil import parser
from typing import Optional

from automon.log import logger

from .container import Container
from .vault import Vault

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class GeneralResponse:
    def __init__(self, results: dict = {}):
        self.__dict__.update(results)

    def __repr__(self):
        return f'{self.__dict__}'


class GenericResponse(GeneralResponse):
    count: int
    num_pages: int
    data: list = None


class AppRunResults(GeneralResponse):
    app: int = None
    app_name: str = None
    app_version: str = None
    action: str = None
    container: int = None
    id: int = None
    message: str = None
    playbook_run: int = None
    status: str = None

    @property
    def asset_name(self):
        name = self.message
        name = name.split(':')[0]
        name = name.split('on asset')[-1]
        name = name.strip()
        name = name.replace('\'', '')

        return name

    def __repr__(self):
        if self.status == 'success':
            return f"{self.app_name} {self.app_version} '{self.asset_name}' '{self.action}'"
        return f"{self.app_name} {self.app_version} '{self.asset_name}' '{self.action}' {self.status}"


class AppRunResponse(GenericResponse):
    data: [AppRunResults] = None

    def summary(self):
        summary = self.__dict__
        summary['data'] = len(summary['data'])
        return f'{summary}'


class CancelPlaybookResponse(GeneralResponse):
    cancelled: int = None
    message: str = None
    playbook_run_id: int = None


class CloseContainerResponse(GeneralResponse):
    id: int = None
    success: bool = None


class CreateContainerAttachmentResponse(GeneralResponse):
    succeeded: bool = None
    message: str = None
    hash: str = None
    vault_id: str = None
    container: int = None
    size: int = None
    id: int = None
    created_via: str = None

    @property
    def vault_id(self):
        return self.hash


class CreateContainerResponse(GeneralResponse):
    success: bool
    id: int = None
    new_artifacts_ids: list


class PlaybookRun(GeneralResponse):
    action_exec: list
    cancelled: Optional[bool]
    container: int = None
    effective_user: int
    id: int = None
    ip_address: str
    last_artifact: int = None
    log_level: int
    message: str = None
    misc: dict = None
    node_guid: Optional[int]
    owner: int = None
    parent_run: Optional[int] = None
    playbook: int = None
    playbook_run_id: Optional[int] = None
    run_data: dict = None
    start_time: str = None
    status: str = None
    test_mode: str = None
    update_time: str = None
    version: int

    def __repr__(self):
        return f'[{self.status}] {self.playbook_name}'

    def __len__(self):
        return self._playbook_run_id

    @property
    def _playbook_run_id(self):
        if self.playbook_run_id:
            return self.playbook_run_id
        return self.id

    @property
    def message_to_dict(self) -> Optional[dict]:
        try:
            return json.loads(self.message)
        except Exception as e:
            log.warning(f'message is not json. {e}')

    @property
    def playbook_name(self):
        if self.message_to_dict:
            return self.message_to_dict['playbook']
        return ''

    @property
    def start_time_parsed(self):
        return parser.parse(self.start_time)

    @property
    def update_time_parsed(self):
        return parser.parse(self.update_time)


class Response(GeneralResponse):
    count: int
    num_pages: int
    data: list = None


class RunPlaybookResponse(GeneralResponse):
    playbook_run_id: str = None
    received: bool


class UpdatePlaybookResponse(GeneralResponse):
    message: int = None
    success: bool = None


class VaultResponse(GeneralResponse):
    count: int = None
    data: list = None
    num_pages: int = None

    def __repr__(self):
        return f'{dict(count=self.count, num_pages=self.num_pages)}'

    @property
    def data_parsed(self):
        return [Vault(x) for x in self.data]

    def get_one(self):
        if self.data_parsed:
            return self.data_parsed[-1]



import unittest

from automon.integrations.seleniumWrapper.browser import SeleniumBrowser
from automon.integrations.seleniumWrapper.config_webdriver_chrome import ChromeWrapper

browser = SeleniumBrowser()
browser.config.webdriver_wrapper = ChromeWrapper()
browser.config.webdriver_wrapper.enable_defaults().enable_headless()


class SeleniumClientTest(unittest.TestCase):
    if browser.run():
        def test_fake_page(self):
            self.assertFalse(browser.get('http://555.555.555.555'))

        def test_real_page(self):
            if browser.get('http://1.1.1.1'):
                self.assertTrue(True)

        def test_screenshot_png(self):
            if browser.get('http://google.com'):
                self.assertTrue(browser.get_screenshot_as_png())

        def test_screenshot_base64(self):
            if browser.get('http://yahoo.com'):
                self.assertTrue(browser.get_screenshot_as_base64())

        def test_screenshot_file(self):
            if browser.get('http://bing.com'):
                self.assertTrue(browser.save_screenshot())
                self.assertTrue(browser.save_screenshot(folder='./'))


if __name__ == '__main__':
    unittest.main()
    browser.quit()



import datetime
import pandas as pd

import unittest

from automon.integrations.google.sheets import GoogleSheetsClient
from automon.integrations.facebook import FacebookGroups


def get_facebook_info(url: str):
    group = FacebookGroups()
    # group.start(headless=False)
    group.start(headless=True)
    group.get(url=url)
    if not group.privacy_details:
        close = group._browser.wait_for(group.xpath_popup_close)
        group._browser.action_click(close)
        about = group._browser.wait_for(group.xpath_about)
        group._browser.action_click(about)

    return group.to_dict()


class MyTestCase(unittest.TestCase):
    def test_authenticate(self):
        spreadsheetId = '1isrvjU0DaRijEztByQuT9u40TaCOCwdaLAXgGmKHap8'
        test = GoogleSheetsClient(
            spreadsheetId=spreadsheetId,
            worksheet='AUDIT list Shelley',
            range='AUDIT list Shelley!A:B'
        )

        if not test.authenticate():
            return

        test.get_values(
            range='AUDIT list Shelley!A:Z',
        )
        test.get(
            ranges='AUDIT list Shelley!A:Z',
            fields="sheets/data/rowData/values/hyperlink",
        )

        data = test.response['sheets'][0]['data'][0]['rowData']
        # expand nested data
        links = []
        for x in data:
            if x:
                links.append(
                    x['values'][0]['hyperlink']
                )

        df = pd.DataFrame(links)

        pass


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.seleniumWrapper.config import SeleniumConfig


class SeleniumConfigTest(unittest.TestCase):
    def test_config(self):
        self.assertTrue(SeleniumConfig())


if __name__ == '__main__':
    unittest.main()



import os
import urllib3

from automon.log import Logging
from automon.helpers import environ

log = Logging(name='MinioConfig', level=Logging.ERROR)


class MinioConfig(object):
    endpoint: str
    access_key: str
    secret_key: str
    session_token: str
    secure: bool
    region: str
    http_client: urllib3.PoolManager

    def __init__(self, endpoint: str = None,
                 access_key: str = None,
                 secret_key: str = None,
                 session_token: str = None,
                 secure: bool = None,
                 region: str = None,
                 http_client: urllib3.PoolManager = None):
        """Minio config"""

        self._endpoint = endpoint or environ('MINIO_ENDPOINT', 'localhost')
        self._access_key = access_key or environ('MINIO_ACCESS_KEY')
        self._secret_key = secret_key or environ('MINIO_SECRET_KEY')
        self._session_token = session_token or environ('MINIO_SESSION_TOKEN')
        self._secure = secure or environ('MINIO_SECURE', True)
        self._region = region or environ('MINIO_REGION')
        self._http_client = http_client or environ('MINIO_HTTP_CLIENT')

        if not self.endpoint:
            log.warning(f'missing MINIO_ENDPOINT')

        if not self.access_key:
            log.warning(f'missing MINIO_ACCESS_KEY')

        if not self.secret_key:
            log.warning(f'missing MINIO_SECRET_KEY')

    @property
    def access_key(self):
        return self._access_key

    @property
    def endpoint(self):
        return self._endpoint

    @property
    def is_ready(self):
        if self.endpoint and self.access_key and self.secret_key:
            return True
        return False

    @property
    def secret_key(self):
        return self._secret_key

    @property
    def secure(self):
        return self._secure

    @property
    def session_token(self):
        return self._session_token

    @property
    def region(self):
        return self._region

    @property
    def http_client(self):
        return self._http_client

    def __repr__(self):
        return f'{self.__dict__}'



import re

from neo4j.work.summary import ResultSummary

from automon.log import logger

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class Results(ResultSummary):
    def __init__(self, results):
        self._results = results
        self.summary = results._summary

        if self.summary:
            self.counters = self.summary.counters
            self.notifications = self.summary.notifications
            self.query = self.summary.query
            self.query_type = self.summary.query_type
            self.metadata = self.summary.metadata

            if self.metadata:
                self.stats = self.metadata.get('stats', None)

    def __str__(self):
        return f'{self.__dict__}'



import unittest

from automon.integrations.splunk_soar import SplunkSoarConfig

c = SplunkSoarConfig()


class TestPhantomConfig(unittest.TestCase):

    def test_config(self):
        if c.is_ready:
            self.assertTrue(c.is_ready)
        else:
            self.assertFalse(c.is_ready)


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.splunk_soar import SplunkSoarClient

c = SplunkSoarClient()


class MyTestCase(unittest.TestCase):
    def test_get_action_run(self):
        if c.is_connected():
            action_run = c.get_action_run(action_run_id=3040610)
            if action_run.id:
                self.assertTrue(True)


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.instagram.client_browser import InstagramBrowserClient


class InstagramClientTest(unittest.TestCase):
    c = InstagramBrowserClient(headless=True)
    c.browser.run()

    if c.is_running():
        c.browser.get(c.urls.domain)
        c.browser.add_cookie_from_base64()
        c.browser.refresh()
        if c.is_authenticated():
            def test_authenticate(self):
                self.assertTrue(self.c.is_authenticated())
                self.c.browser.quit()


if __name__ == '__main__':
    unittest.main()



import random

from automon.log import logger

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class SeleniumUserAgentBuilder:
    googlebot = [
        'Mozilla/5.0 AppleWebKit/537.36 (KHTML, like Gecko; compatible; Googlebot/2.1; +http://www.google.com/bot.html) Chrome/W.X.Y.Z Safari/537.36',
    ]

    top = [
        'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36 Edg/119.0.0.0',
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36',
    ]

    macox = [
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 13_3_1) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.4 Safari/605.1.15',
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 13.3; rv:112.0) Gecko/20100101 Firefox/112.0',
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 13_3_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36',
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 13_3_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36 Vivaldi/5.7.2921.68',
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 13_3_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36 Edg/112.0.1722.48',
    ]
    windows = [
        'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36 Edg/112.0.1722.48',
        'Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; rv:11.0) like Gecko',
        'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36',
        'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36',
        'Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36',
        'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:112.0) Gecko/20100101 Firefox/112.0',
        'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36 Vivaldi/5.7.2921.68',
        'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36 Vivaldi/5.7.2921.68',
    ]

    agents = [

    ]
    agents.extend(googlebot)
    agents.extend(top)
    agents.extend(macox)
    agents.extend(windows)

    def filter_agent(self, filter: list or str, case_sensitive: bool = False) -> list:
        if isinstance(filter, str):
            filter = [filter]

        filtered_agents = []
        for agent in self.agents:
            if not case_sensitive:
                if any(substring.lower() in agent.lower() for substring in filter):
                    filtered_agents.append(agent)
            else:
                if any(substring in agent for substring in filter):
                    filtered_agents.append(agent)

        return filtered_agents

    def get_random_agent(self, filter: list or str = None, **kwargs):
        if filter:
            filtered_agents = self.filter_agent(filter, **kwargs)
            if filtered_agents:
                return random.choice(filtered_agents)
            else:
                return ''

        return random.choice(self.agents)

    def pick_random(self, choices: list):
        return random.choice(choices)

    def get_mac(self, **kwargs):
        return self.get_random_agent(filter='Macintosh', **kwargs)

    def get_top(self, **kwargs):
        return self.pick_random(self.top)

    def get_windows(self, **kwargs):
        return self.get_random_agent(filter='Windows', **kwargs)



import unittest

from automon.helpers.grok import Grok, GrokLegacy


class GrokTest(unittest.TestCase):

    def test_Grok(self):
        self.assertTrue(Grok())


class GrokLegacyTest(unittest.TestCase):

    def test_GrokLegacy(self):
        self.assertTrue(GrokLegacy())


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.splunk import SplunkConfig

config_mock = SplunkConfig(
    host='localhost',
    username='user',
    password='pass')

config_cloud = SplunkConfig(
    host='splunkcloud.com',
    username='user',
    password='pass')


class SplunkConfigTest(unittest.TestCase):
    config = SplunkConfig()

    def test_init(self):
        self.assertIsNotNone(SplunkConfig())

    def test_print(self):
        self.assertTrue(f'{self.config}')

    def test_info(self):
        self.assertTrue(self.config.info())


if __name__ == '__main__':
    unittest.main()



import unittest

from flask import Flask

from automon.integrations.flaskWrapper.boilerplate import FlaskBoilerplate
from automon.integrations.flaskWrapper.config import FlaskConfig


class FlaskTest(unittest.TestCase):
    app = Flask(__name__)

    def test_FlaskBoilerplate(self):
        self.assertTrue(FlaskBoilerplate)
        self.assertTrue(FlaskBoilerplate())

    def test_ConfigFlask(self):
        self.assertTrue(FlaskConfig.javascript_compatibility(self.app))
        self.assertTrue(FlaskConfig.hash_key('blob'))
        self.assertTrue(FlaskConfig.new_secret_key())

# if __name__ == '__main__':
#     unittest.main()



class LogRecordAttribute(object):

    def __init__(self, timestamp: bool = False):
        self._log_pattern = []

        if timestamp:
            self.asctime()

    def __repr__(self):
        return f'\t'.join(self._log_pattern)

    def __str__(self):
        return f'\t'.join(self._log_pattern)

    def ALL(self):
        self._log_pattern.append('[asctime]')
        self.asctime()
        self._log_pattern.append('[created]')
        self.created()
        self._log_pattern.append('[filename]')
        self.filename()
        self._log_pattern.append('[funcName]')
        self.funcName()
        self._log_pattern.append('[levelname]')
        self.levelname()
        self._log_pattern.append('[levelno]')
        self.levelno()
        self._log_pattern.append('[lineno]')
        self.lineno()
        self._log_pattern.append('[message]')
        self.message()
        self._log_pattern.append('[module]')
        self.module()
        self._log_pattern.append('[msecs]')
        self.msecs()
        self._log_pattern.append('[name]')
        self.name()
        self._log_pattern.append('[pathname]')
        self.pathname()
        self._log_pattern.append('[process]')
        self.process()
        self._log_pattern.append('[processName]')
        self.processName()
        self._log_pattern.append('[relativeCreated]')
        self.relativeCreated()
        self._log_pattern.append('[thread]')
        self.thread()
        self._log_pattern.append('[threadName]')
        self.threadName()
        # self._log_pattern.append('[taskName]')
        # self.taskName()

        return self

    def asctime(self):
        self._log_pattern.append('%(asctime)s')
        return self

    def created(self):
        self._log_pattern.append('%(created)f')
        return self

    def filename(self):
        self._log_pattern.append('%(filename)s')
        return self

    def funcName(self):
        self._log_pattern.append('%(funcName)s')
        return self

    def funcName_and_levelno(self):
        self._log_pattern.append('%(funcName)s:%(lineno)s')
        return self

    def levelname(self):
        self._log_pattern.append('%(levelname)s')
        return self

    def levelno(self):
        self._log_pattern.append('%(levelno)s')
        return self

    def lineno(self):
        self._log_pattern.append('%(lineno)d')
        return self

    def message(self):
        self._log_pattern.append('%(message)s')
        return self

    def module(self):
        self._log_pattern.append('%(module)s')
        return self

    def msecs(self):
        self._log_pattern.append('%(msecs)d')
        return self

    def name(self):
        self._log_pattern.append('%(name)s')
        return self

    def name_and_lineno(self):
        self._log_pattern.append('%(name)s:%(lineno)s')
        return self

    def pathname(self):
        self._log_pattern.append('%(pathname)s')
        return self

    def process(self):
        self._log_pattern.append('%(process)d')
        return self

    def processName(self):
        self._log_pattern.append('%(processName)s')
        return self

    def relativeCreated(self):
        self._log_pattern.append('%(relativeCreated)d')
        return self

    def thread(self):
        self._log_pattern.append('%(thread)d')
        return self

    def threadName(self):
        self._log_pattern.append('%(threadName)s')
        return self

    def taskName(self):
        self._log_pattern.append('%(taskName)s')
        return self



from automon.log import Logging

log = Logging(name='MinioAssertions', level=Logging.DEBUG)


class MinioAssertions:

    @staticmethod
    def bucket_name(bucket: str) -> bool:
        if bucket == f'{bucket}'.lower():
            return f'{bucket}'
        log.warning(msg=f'bucket name "{bucket}" must be lower')
        return f'{bucket}'.lower()



import unittest

from automon.integrations.slackWrapper.client import SlackClient
from automon.integrations.slackWrapper.error import SlackError
from automon.integrations.slackWrapper.bots import BotInfo
from automon.integrations.slackWrapper.config import SlackConfig
from automon.integrations.slackWrapper.slack_formatting import Format, Chat, Emoji


class ConfigTest(unittest.TestCase):
    def test_SlackConfig(self):
        self.assertTrue(SlackConfig())


class ClientTest(unittest.TestCase):

    def test_Slack(self):
        self.assertTrue(SlackClient())

    def test_SlackError(self):
        self.assertTrue(SlackError)

    def test_Format(self):
        self.assertTrue(Format.blockquote)
        self.assertTrue(Format.codeblock)


class BotTest(unittest.TestCase):
    def test_BotInfo(self):
        bot = {
            "ok": True,
            "bot": {
                "id": "B061F7JD2",
                "deleted": False,
                "name": "beforebot",
                "updated": 1449272004,
                "app_id": "A161CLERW",
                "user_id": "U012ABCDEF",
                "icons": {
                    "image_36": "https://...",
                    "image_48": "https://...",
                    "image_72": "https://..."
                }
            }
        }

        self.assertTrue(BotInfo)
        self.assertTrue(BotInfo(bot))
        self.assertEqual(BotInfo(bot), BotInfo(bot))
        self.assertNotEqual(BotInfo(bot), bot)
        self.assertTrue(f'{BotInfo(bot)}')


class FormatTest(unittest.TestCase):

    def test_Chat(self):
        self.assertTrue(Chat.Format('test', Format.codeblock))
        self.assertTrue(Chat.wrap('test', Format.blockquote))
        self.assertFalse(Chat.none())
        self.assertTrue(Chat.clean('test'))
        self.assertFalse(Chat.clean(None))
        self.assertTrue(Chat.string('test'))
        self.assertFalse(Chat.string(None))
        self.assertFalse(Chat.multi_to_single_line(None))
        self.assertFalse(Chat.multi_to_single_line('\n'))
        self.assertTrue(Chat.multi_to_single_line('test'))
        self.assertTrue(Chat.multi_to_single_line('test\n'))
        self.assertTrue(Chat.multi_to_single_line('test\ntest'))

    def test_Emoji(self):
        self.assertTrue(Emoji.warning)
        self.assertTrue(Emoji.information_source)
        self.assertTrue(Emoji.magnifying_glass)
        self.assertTrue(Emoji.sos)
        self.assertTrue(Emoji.waiting)
        self.assertTrue(Emoji.still_waiting)
        self.assertTrue(Emoji.grey_question)
        self.assertTrue(Emoji.questionbang)
        self.assertTrue(Emoji.skull)
        self.assertTrue(Emoji.skull_and_crossbones)
        self.assertTrue(Emoji.file)
        self.assertTrue(Emoji.announcement)
        self.assertTrue(Emoji.yay)


if __name__ == '__main__':
    unittest.main()



import io
import minio
import socket
import functools

from typing import Optional

from automon.log import Logging

from .bucket import Bucket
from .object import Object, DeleteObject
from .config import MinioConfig
from .assertions import MinioAssertions

log = Logging(name='MinioClient', level=Logging.DEBUG)


class MinioClient(object):
    endpoint: str
    access_key: str
    secret_key: str
    config: MinioConfig

    def __init__(self,
                 endpoint: str = None,
                 access_key: str = None,
                 secret_key: str = None,
                 config: MinioConfig = None):
        """Minio client
        """

        self.config = config or MinioConfig(endpoint=endpoint, access_key=access_key, secret_key=secret_key)

    @property
    def client(self):
        client = minio.Minio(
            endpoint=self.config.endpoint,
            access_key=self.config.access_key,
            secret_key=self.config.secret_key,
            session_token=self.config.session_token,
            secure=self.config.secure,
            region=self.config.region,
            http_client=self.config.http_client
        )
        return client

    def _is_connected(func):
        """Decorator that checks if MinioClient can list buckets
        """

        @functools.wraps(func)
        def _wrapper(self, *args, **kwargs):
            if not self.config.is_ready:
                return False
            try:
                self.client.list_buckets()
                # if not self._sessionExpired() or self.client.list_buckets():
                return func(self, *args, **kwargs)
            except Exception as e:
                log.error(f'Minio client not connected. {e}')
            return False

        return _wrapper

    @_is_connected
    def download_object(self, bucket_name: str, file: str):
        """Minio object downloader
        """
        bucket_name = MinioAssertions.bucket_name(bucket_name)

        log.debug(f'[downloader] Downloading: {bucket_name}/{file.object_name}')
        return self.client.get_object(bucket_name, file.object_name)

    @_is_connected
    def get_bucket(self, bucket_name: str, **kwargs) -> Bucket or None:
        """List Minio buckets
        """
        bucket_name = MinioAssertions.bucket_name(bucket_name)
        buckets = self.list_buckets(**kwargs)

        if bucket_name in buckets:
            bucket_index = buckets.index(bucket_name)
            return buckets[bucket_index]

        log.info(msg=f'Get bucket: "{bucket_name}" does not exist')
        return

    @_is_connected
    def is_connected(self):
        """Check if MinioClient is connected
        """
        log.info(f'Minio client connected')
        return True

    @_is_connected
    def list_buckets(self, **kwargs) -> [Bucket]:
        """List Minio buckets
        """
        buckets = self.client.list_buckets(**kwargs)
        buckets = [Bucket(x) for x in buckets]

        log.info(f'List buckets: {len(buckets)}')
        return buckets

    @_is_connected
    def list_objects(
            self,
            bucket_name: str,
            prefix: str = None,
            recursive: bool = False,
            start_after: str = None, **kwargs) -> [Object]:
        """List Minio objects
        """
        bucket_name = MinioAssertions.bucket_name(bucket_name)

        try:
            objects = self.client.list_objects(
                bucket_name=bucket_name,
                prefix=prefix,
                recursive=recursive,
                start_after=start_after, **kwargs)
            objects = [Object(x) for x in objects]

            msg = f'List objects total: {len(objects)} (bucket: "{bucket_name}")'

            if prefix:
                msg += f' Prefix: "{prefix}"'

            log.info(msg)
            return objects

        except Exception as error:
            log.error(f'failed to list objects. {error}', enable_traceback=False)

        return []

    @_is_connected
    def list_objects_generator(
            self,
            bucket_name: str,
            folder: str = None,
            recursive: bool = True, **kwargs) -> [Object]:
        """Generator for Minio objects
        """
        bucket_name = MinioAssertions.bucket_name(bucket_name)

        try:
            objects = self.client.list_objects(bucket_name, folder, recursive=recursive, **kwargs)
            return objects

        except Exception as e:
            log.error(f'failed to list objects. {e}')

        return []

    @_is_connected
    def remove_bucket(self, bucket_name: str, **kwargs) -> bool:
        """Remove bucket
        """
        bucket_name = MinioAssertions.bucket_name(bucket_name)

        try:
            self.client.remove_bucket(bucket_name, **kwargs)
            log.info(f'Removed bucket: "{bucket_name}"')
            return True

        except Exception as e:
            log.error(f'Remove bucket: "{bucket_name}" failed. {e}', enable_traceback=False)

        return False

    @_is_connected
    def remove_objects(self, bucket_name: str, prefix: str = None, **kwargs) -> bool:
        """Remove all objects
        """
        bucket_name = MinioAssertions.bucket_name(bucket_name)
        objects = self.list_objects(bucket_name=bucket_name, prefix=prefix)
        delete_objects = [DeleteObject(x) for x in objects]

        if not delete_objects:
            log.info(f'Bucket is empty: "{bucket_name}"')
            return True

        errors = list(self.client.remove_objects(bucket_name, delete_objects, **kwargs))
        log.info(f'Removed {len(delete_objects)} objects in bucket "{bucket_name}"')

        if self.list_objects(bucket_name, prefix):
            return self.remove_objects(bucket_name, prefix=prefix)

        return True

    @_is_connected
    def make_bucket(self, bucket_name: str) -> Bucket:
        """Make a bucket
        """
        bucket_name = MinioAssertions.bucket_name(bucket_name)
        try:
            self.client.make_bucket(bucket_name)
            log.info(f'Created bucket: "{bucket_name}"')

        except Exception as e:
            log.warning(f'Bucket exists: "{bucket_name}". {e}')

        return self.get_bucket(bucket_name)

    @_is_connected
    def put_object(self, bucket_name: str, object_name: str, data: io.BytesIO, length: int = None,
                   content_type='application/octet-stream',
                   metadata=None, sse=None, progress=None,
                   part_size=None):
        """Minio object uploader
        """
        bucket_name = MinioAssertions.bucket_name(bucket_name)
        length = length or data.getvalue().__len__()

        log.debug(f'[{self.put_object.__name__}] Uploading: {object_name}')

        try:
            put = self.client.put_object(
                bucket_name=bucket_name,
                object_name=object_name,
                data=data, length=length,
                content_type=content_type,
                metadata=metadata, sse=sse,
                progress=progress)

            log.info(
                f'[put_object] Saved to: '
                f'{self.config.endpoint}/{bucket_name}/{object_name}'
            )

            return put

        except Exception as e:
            log.error(
                f'[{self.put_object.__name__}] Unable to save: '
                f'{self.config.endpoint}/{bucket_name}/{bucket_name} '
                f'{e}',
                raise_exception=False
            )

            return False

    def __repr__(self):
        return f'endpoint: {self.config.endpoint}'


def check_connection(host, port):
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(1)
        s.connect((host, port))
        s.close()
        return True
    except Exception as e:
        log.error(e, enable_traceback=False)
        return False



import os


def environ(env_var: str, default: any = None) -> bool or str or None:
    """Get environment variable, else return default"""
    env = os.getenv(env_var)
    if env:
        if f'{env}'.lower() == 'true':
            return True
        if f'{env}'.lower() == 'false':
            return False
        return f'{env}'.strip()
    return default



import psutil

from automon.log import Logging

log = Logging('cpu', level=Logging.DEBUG)


def cpu_usage(max_cpu_percentage=80):
    """Limit max cpu usage
    """
    if psutil.cpu_percent() < max_cpu_percentage:
        log.debug(f'[{cpu_usage.__name__}] {psutil.cpu_percent()}%')
        return True
    else:
        log.debug(f'[{cpu_usage.__name__}] {psutil.cpu_percent()}%')
        return False



class Geoip:
    """With the given IP address, retrieve geoip information
    """

    def __init__(self, ip: str):
        self.ip = ip

    # TODO: parse geoiptool.com
    def geoiptool(self):
        pass



import unittest

from automon.integrations.splunk_soar import SplunkSoarClient

c = SplunkSoarClient()


class TestClient(unittest.TestCase):
    if c.is_connected():
        def test_list_vault_generator(self):
            filter = 'ca3f4b65155db20d6e1d3b5fee8ef8bf0d968548'
            test = c.filter_vault(filter=filter, page_size=200)
            if test:
                self.assertTrue(True)


if __name__ == '__main__':
    unittest.main()



from selenium.webdriver.common.desired_capabilities import DesiredCapabilities

caps = DesiredCapabilities.CHROME
 #as per latest docs
caps['goog:loggingPrefs'] = {'performance': 'ALL'}
driver = webdriver.Chrome(desired_capabilities=caps)


class SeleniumDesiredCapabilities(DesiredCapabilities):

    def __init__(self):
        pass

    @property
    def DesiredCapabilities(self):
        return DesiredCapabilities

    @property
    def CHROME(self):
        return self.DesiredCapabilities.CHROME


import unittest

from automon.log import logger

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class LoggingTest(unittest.TestCase):

    def test_logger(self):
        self.assertTrue(log)

    def test_error(self):
        self.assertIsNone(log.error('test'))

    def test_debug(self):
        self.assertIsNone(log.debug('test'))

    def test_info(self):
        self.assertIsNone(log.info('test'))

    def test_critical(self):
        self.assertIsNone(log.critical('test'))

    def test_warn(self):
        self.assertIsNone(log.warning('test'))


if __name__ == '__main__':
    unittest.main()



import unittest
import warnings

from automon.integrations.splunk_soar import SplunkSoarClient

c = SplunkSoarClient()


class TestClient(unittest.TestCase):
    if c.is_connected():
        def test_list_app_run_by_playbook_run(self):
            # TODO: create test list_app_run_by_playbook_run
            pass


if __name__ == '__main__':
    unittest.main()



import unittest
import datetime
import hashlib

from automon.integrations.minioWrapper import MinioClient
from automon.integrations.minioWrapper.bucket import Bucket

MINIO_ENDPOINT = 'play.minio.io:9000'
MINIO_ACCESS_KEY = 'Q3AM3UQ867SPQQA43P2F'
MINIO_SECRET_KEY = 'zuf+tfteSlswRu7BJ86wekitnifILbZam1KYY3TG'

c = MinioClient(endpoint=MINIO_ENDPOINT, access_key=MINIO_ACCESS_KEY, secret_key=MINIO_SECRET_KEY)
bucket = hashlib.md5(f'{datetime.datetime.now()}'.encode()).hexdigest()


class ClientTest(unittest.TestCase):

    def test_remove_bucket(self):
        if c.is_connected():
            test = c.make_bucket(bucket)
            bucket_name = test.name
            c.remove_objects(bucket_name)
            self.assertTrue(c.remove_bucket(bucket_name))


if __name__ == '__main__':
    unittest.main()



class XPaths(object):

    def __repr__(self):
        return 'Instagram XPaths'

    @property
    def login_user(self):
        return [
            '/html/body/div[2]/div/div/div[1]/div/div/div/div[1]/section/main/div/div/div[1]/div[2]/form/div/div[1]/div/label/input',
            '/html/body/div[2]/div/div/div[2]/div/div/div/div[1]/section/main/div/div/div[1]/div[2]/form/div/div[1]/div/label/input',
            '/html/body/div[2]/div/div/div/div[1]/div/div/div/div[1]/section/main/div/div/div[1]/div[2]/form/div/div[1]/div/label/input',
        ]

    @property
    def login_pass(self):
        return [
            '/html/body/div[2]/div/div/div[2]/div/div/div/div[1]/section/main/div/div/div[1]/div[2]/form/div/div[2]/div/label/input',
            '/html/body/div[2]/div/div/div[1]/div/div/div/div[1]/section/main/div/div/div[1]/div[2]/form/div/div[2]/div/label/input',
            '/html/body/div[2]/div/div/div/div[1]/div/div/div/div[1]/section/main/div/div/div[1]/div[2]/form/div/div[2]/div/label/input'
        ]

    @property
    def login_btn(self):
        return [
            '/html/body/div[2]/div/div/div[2]/div/div/div/div[1]/section/main/div/div/div[1]/div[2]/form/div/div[3]/button',
            '/html/body/div[2]/div/div/div[1]/div/div/div/div[1]/section/main/div/div/div[1]/div[2]/form/div/div[3]/button',
            '/html/body/div[2]/div/div/div[1]/div/div/div/div[1]/section/main/div/div/div[1]/div[2]/form/div/div[4]/button',
            '/html/body/div[2]/div/div/div/div[1]/div/div/div/div[1]/section/main/div/div/div[1]/div[2]/form/div/div[3]/button'
        ]

    @property
    def authenticated_paths(self):
        authenticated = []
        authenticated.extend(self.profile_picture)
        authenticated.extend(self.home)
        return authenticated

    @property
    def home(self):
        return [
            '/html/body/div[2]/div/div/div[2]/div/div/div/div[1]/div[1]/div[1]/div/div/div/div/div[2]/div[1]/div/div/a/div',
        ]

    @property
    def profile_picture(self):
        return [
            '/html/body/div[2]/div/div/div[2]/div/div/div/div[1]/div[1]/div[2]/section/main/div[1]/div[2]/div/div[1]/div/div/div/div/div/div[1]/div/div/a/img',
            '/html/body/div[2]/div/div/div[2]/div/div/div/div[1]/div[1]/div[1]/div/div/div/div/div[2]/div[8]/div/span/div/a/div/div[1]/div/div/span/img',
            '/html/body/div[2]/div/div/div[2]/div/div/div/div[1]/div[1]/div[1]/div/div/div/div/div/div[6]/div/span/div/a/div/div/div/div/span/img',
            '/html/body/div[2]/div/div/div[2]/div/div/div/div[1]/div[1]/div[2]/section/main/div[1]/section/div[3]/div[1]/div/div/div/div/div/div[1]/div/div/span/img',
            '/html/body/div[2]/div/div/div[2]/div/div/div/div[1]/div[1]/div[2]/section/main/div[1]/section/div/div[2]/div/div/div/div/ul/li[3]/div/button/div[1]/span/img',
        ]

    @property
    def save_info(self):
        return '/html/body/div[2]/div/div/div/div[1]/div/div/div/div[1]/section/main/div/div/div/section/div/button'

    @property
    def save_info_not_now_div(self):
        return [
            '/html/body/div[2]/div/div/div[3]/div/div/div[1]/div/div[2]/div/div/div/div/div[2]/div/div',
        ]

    @property
    def save_info_not_now(self):
        return [
            '/html/body/div[2]/div/div/div[3]/div/div/div[1]/div/div[2]/div/div/div/div/div[2]/div/div/div[3]/button[2]',
            '/html/body/div[2]/div/div/div[2]/div/div/div/div[1]/div[1]/div[2]/section/main/div/div/div/div',
            '/html/body/div[2]/div/div/div/div[1]/div/div/div/div[1]/section/main/div/div/div/div/button'
        ]

    @property
    def turn_on_notifications(self):
        return [
            '/html/body/div[2]/div/div/div[3]/div/div/div[1]/div/div[2]/div/div/div/div/div[2]/div/div/div[3]/button[1]',
            '/html/body/div[2]/div/div/div/div[2]/div/div/div[1]/div/div[2]/div/div/div/div/div[2]/div/div/div[3]/button[1]'
        ]

    @property
    def turn_on_notifications_not_now(self):
        return [
            '/html/body/div[3]/div[1]/div/div[2]/div/div/div/div/div[2]/div/div/div[3]/button[2]',
            '/html/body/div[2]/div/div/div[3]/div/div/div[1]/div/div[2]/div/div/div/div/div[2]/div/div/div[3]/button[2]',
            '/html/body/div[2]/div/div/div/div[2]/div/div/div[1]/div/div[2]/div/div/div/div/div[2]/div/div/div[3]/button[2]'
        ]



import unittest

from automon.integrations.shodan import Shodan, ShodanConfig


class ShodanTest(unittest.TestCase):
    def test_Shodan(self):
        self.assertIsNotNone(Shodan())
        self.assertIsNone(Shodan().request())

    def test_ShodanConfig(self):
        self.assertTrue(ShodanConfig())

# if __name__ == '__main__':
#     unittest.main()



import sys
import unittest

from automon.integrations.mac.airport import Airport


class AirportTest(unittest.TestCase):
    a = Airport()

    def test_run(self):
        if self.a.isReady():
            self.assertTrue(self.a.run())

    def test_scan(self):
        if self.a.isReady():
            self.assertTrue(self.a.scan())
            self.assertTrue(self.a.scan(0))

    def test_summary(self):
        if self.a.isReady():
            self.assertTrue(self.a.scan_summary())
            self.assertTrue(self.a.scan_summary(0))

    def test_xml(self):
        if self.a.isReady():
            scan = self.a.scan_xml()
            if scan:
                self.assertTrue(scan)
            self.assertFalse(self.a.scan_xml(0))

    def test_set_channel(self):
        if self.a.isReady():
            pass
            # self.assertTrue(self.a.set_channel(10))

    def test_disassociate(self):
        if self.a.isReady():
            pass
            # self.assertTrue(self.a.disassociate())

    def test_getinto(self):
        if self.a.isReady():
            self.assertTrue(self.a.getinfo())

    def test_create_psk(self):
        if self.a.isReady():
            self.assertTrue(self.a.create_psk(ssid='AAAAAAAA', passphrase='CALVIN'))


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.neo4jWrapper.config import Neo4jConfig


class ConfigTest(unittest.TestCase):
    c = Neo4jConfig()

    def test_config(self):
        self.assertTrue(Neo4jConfig)

    def test_NEO4J_USER(self):
        self.assertEqual(type(self.c.NEO4J_USER), str)

    def test_NEO4J_PASSWORD(self):
        self.assertEqual(type(self.c.NEO4J_PASSWORD), str)

    def test_NEO4J_SERVERS(self):
        self.assertEqual(type(self.c.NEO4J_HOST), str)


if __name__ == '__main__':
    unittest.main()



import requests
import elasticsearch

from datetime import datetime
from requests.auth import HTTPBasicAuth
from elasticsearch import Elasticsearch

from automon.log import Logging
from .config import ElasticsearchConfig
from automon.helpers.sanitation import Sanitation


class ElasticsearchClient(Elasticsearch):
    def __init__(self, host: str = None,
                 cloud_id: str = None,
                 user: str = None,
                 password: str = None,
                 api_key: tuple = None,
                 api_key_id: str = None,
                 api_key_secret: str = None,
                 config: ElasticsearchConfig = None):
        """elasticsearch wrapper

        :param host: str
        :param cloud_id: str
        :param user: str
        :param password: str
        :param api_key: base64 str
        :param api_key_id: str
        :param api_key_secret: str
        :param config: class ElasticsearchConfig
        """

        self._log = Logging(ElasticsearchClient.__name__, Logging.DEBUG)

        self.config = config or ElasticsearchConfig(
            host=host,
            cloud_id=cloud_id,
            user=user,
            password=password,
            api_key=api_key,
            api_key_id=api_key_id,
            api_key_secret=api_key_secret)

        self.client = self._client()

        self.indices = []
        self.success = []
        self.errors = []
        self.results = None

    def __repr__(self):
        return f'{self.client}, indices: {self.indices}, errors: {self.errors}, {self.config}'

    def _client(self):
        """Elasticsearch client"""
        try:
            client = Elasticsearch(
                hosts=self.config.ELASTICSEARCH_HOST,
                cloud_id=self.config.ELASTICSEARCH_CLOUD_ID,
                api_key=self.config.ELASTICSEARCH_API_KEY,
                request_timeout=self.config.ELASTICSEARCH_REQUEST_TIMEOUT,
                http_auth=self.config.http_auth,
                verify_certs=self.config.verify_certs
            )
            self._log.info(f'Connected to elasticsearch: {client}')
            return client

        except Exception as e:
            self._log.error(f'Cannot connect to elasticsearch: {self.config.ELASTICSEARCH_HOST}, {e}', enable_traceback=False)

        return False

    def connected(self):
        """Check if connected"""
        if self.client:
            try:
                if self.client.ping():
                    return True
            except Exception as e:
                self._log.error(f'{e}', enable_traceback=False)
        return False

    def create_document(self, doc: dict, index: str = 'default', id: str = None):
        """Create document

        :param doc:
        :param index:
        :param id:
        :return: bool
        """
        # doc = {
        #     'author': 'kimchy',
        #     'text': 'Elasticsearch: cool. bonsai cool.',
        #     'timestamp': datetime.now(),
        # }

        try:
            self.results = self.client.index(index=index, body=doc, id=id)

            r = self.results
            self.client.indices.refresh(index=index)
            self.success.append({'doc': doc, 'index': index, 'id': id, 'result': r})
            self._log.debug(f'created document: {index} {id} {doc}')
            return True
        except Exception as e:
            self._log.error(f'Create document failed: {e}')
            self.errors.append({'index': index, 'doc': doc, 'id': id, 'error': e})
        return False

    def delete_index(self, index: str, **kwargs):
        if self.connected():
            try:
                self.results = self.client.indices.delete(index=index, ignore=[400, 404], **kwargs)

                r = self.results
                self.success.append({'delete index': index, 'result': r})
                self._log.debug(f'deleted index: {index}')
                return True
            except Exception as e:
                self.errors.append({'index': index, 'error': e})
                self._log.error(f'Delete index failed: {e}')

        return False

    def delete_document(self, index: str, id: str = None):
        if self.connected():
            try:
                self.results = self.client.delete(index=index, id=id, ignore=[400, 404])

                r = self.results
                self.success.append({'index': index, 'id': id, 'result': r})
                self._log.debug(f'deleted document: {index} {id}')
                return True
            except Exception as e:
                self._log.error(f'Delete document failed: {e}')
                self.errors.append({'index': index, 'id': id, 'error': e})

        return False

    def get_indices(self) -> bool:
        if self.connected():
            try:
                self.results = self.client.indices.get('*')

                retrieved_indices = self.results
                self.indices = retrieved_indices
                self._log.info(f'Retrieved {len(retrieved_indices)} indices')

                for i in retrieved_indices:
                    info = retrieved_indices.get(i)
                    date = int(info.get('settings').get('index').get('creation_date')) / 1000.0
                    date = datetime.fromtimestamp(date).strftime("%A, %B %d, %Y %I:%M:%S")
                    self._log.debug(f'Index: (created: {date})\t{i}')

                self.success.append({'indices': retrieved_indices})
                self._log.info(f'indices: {len(retrieved_indices)}')
                return True
            except Exception as e:
                self._log.error(f'Failed to get indices: {e}')
                self.errors.append({'error': e})

        return False

    def info(self) -> bool:

        try:
            self.results = self.client.info()
            return True
        except Exception as e:
            self._log.error(f'Failed to get info:{e}')
            self.errors.append({'error': e})

        return False

    def isConnected(self):
        return self.connected()

    def ping(self):
        if self.connected():
            try:
                self.client.ping()
                self._log.debug(f'Ping successful')
                return True
            except Exception as e:
                self.errors.append({'error': e})
                self._log.error(f'Ping failed: {e}')

        return False

    def rest(self, url: str) -> requests:
        try:
            if self.config.ELASTICSEARCH_USER and self.config.ELASTICSEARCH_PASSWORD:
                self.results = requests.get(url, auth=HTTPBasicAuth(
                    self.config.ELASTICSEARCH_USER,
                    self.config.ELASTICSEARCH_PASSWORD
                ))
                r = self.results
            else:
                self.results = requests.get(url)
                r = self.results

            self.success.append({'url': url, 'result': r})
            return r

        except Exception as e:
            self._log.error(f'REST request failed: {e}')
            self.errors.append({'url': url, 'error': e})

        return False

    def search(self, search: dict = None, index: str = 'default'):

        if not search:
            search = {"query": {"match_all": {}}}

        try:
            self.results = self.client.search(index=index, body=search)

            r = self.results
            self.success.append({'search': search, 'index': index, 'result': r})
            self._log.debug(f'search :{search} {index}, result {r}')
            return True
        except Exception as e:
            self._log.error(f'Search failed: {e}')
            self.errors.append({'search': search, 'index': index, 'error': e})
        return False

    def search_summary(self, **kwargs):

        self.results = self.search(kwargs)

        res = self.results
        print(f"Got {res['hits']['total']['value']} Hits")

        for hit in res['hits']['hits']:
            print(f'{hit.get("_source")}')

        self.success.append({'result': res})
        self._log.debug(f'search summary {res}')

        return True

    # def delete_indices(self, index_pattern):
    #     """Requires user interaction"""
    #
    #     if not self.connected:
    #         return False
    #
    #     retrieved_indices = [x for x in self.client.search_indices(index_pattern).keys()]
    #     num_indices = len(retrieved_indices)
    #
    #     if not num_indices:
    #         self._log.debug(f'No indices found')
    #         return False
    #
    #     self._log.info(f'Search found {num_indices} indices')
    #
    #     for index in retrieved_indices:
    #         self._log.debug(index)
    #
    #     # TODO: Find a way to undo index deletions
    #     #       One way could be to rename the indices and store a link to the new
    #     #       indices in a node of deleted indices
    #     if num_indices < 2:
    #         msg = f"\nYOU'RE ABOUT TO DELETE {num_indices} INDEX! ARE YOU SURE YOU WANT TO CONTINUE? "
    #     elif num_indices > 1:
    #         msg = f"\nYOU'RE ABOUT TO DELETE {num_indices} INDICES! ARE YOU SURE YOU WANT TO CONTINUE? "
    #     msg += 'THIS CANNOT BE UNDONE! DECIDED WISELY [y/N]'
    #     print(msg)
    #
    #     answer = str(input()).lower()
    #
    #     if not answer:
    #         answer = 'N'
    #
    #     if answer == 'y':
    #         for index in retrieved_indices:
    #             msg = f'Deleting {index}...'
    #             print(msg, end='')
    #             # Delete the index
    #             self.delete_index(index)
    #             print('done')
    #     else:
    #         print('Whew, you might have just blew it, if you had said yes')

    def search_indices(self, index_pattern):
        if self.connected() and index_pattern:
            try:
                self.results = self.client.indices.get(index_pattern)

                retrieved_indices = self.results
                num_indices = len(retrieved_indices)
                self._log.info(f'Search found {num_indices} indices')
                self.success.append({'index pattern': index_pattern, 'result': retrieved_indices})
                self._log.debug(f'search indices: {index_pattern}')
                return True

            except elasticsearch.exceptions.NotFoundError as e:
                self._log.error(
                    f"You provided the index pattern '{index_pattern}', but returned no results")
                self.errors.append({'index pattern': index_pattern, 'error': e})
            except Exception as e:
                self._log.error(f'Failed to search indices: {e}')
                self.errors.append({'index pattern': index_pattern, 'error': e})

        return False



import pandas

from automon.log import Logging


def Series(*args, **kwargs) -> pandas.Series:
    log = Logging('Series', level=Logging.ERROR)
    s = pandas.Series(*args, **kwargs)
    log.debug(s)
    return s



from .client import RequestsClient
from .config import RequestsConfig

from automon.log import logger

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)

class BaseRestClient:
    requests: RequestsClient
    config: RequestsConfig

    def __init__(self, config: RequestsConfig = None):
        """Base REST Client"""
        self.config = config or RequestsConfig()
        self.requests = RequestsClient()

    def isConnected(self):
        if self.config.is_ready:
            return True
        return False

    def get(self, url: str, data: str = None, headers: dict = None) -> bool:
        return self.requests.get(url=url, data=data, headers=headers)

    def post(self, url: str, data: str = None, headers: dict = None) -> bool:
        return self.requests.post(url=url, data=data, headers=headers)

    def delete(self, url: str, data: str = None, headers: dict = None) -> bool:
        return self.requests.delete(url=url, data=data, headers=headers)

    def patch(self, url: str, data: str = None, headers: dict = None) -> bool:
        return self.requests.patch(url=url, data=data, headers=headers)

    def __repr__(self):
        return f'{self.__dict__}'



from automon.log import logger
from automon.integrations.google.auth import GoogleAuthClient

from .config import GoogleSheetsConfig

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class Fields:
    hyperlink: str = 'sheets/data/rowData/values/hyperlink'


class ValueInputOption:
    USER_ENTERED: str = 'USER_ENTERED'
    RAW: str = 'RAW'


class GoogleSheetsClient(GoogleAuthClient):
    """Google Sheets client"""

    spreadsheetId: str
    worksheet: str
    range: str
    config: GoogleSheetsConfig

    def __init__(
            self,
            spreadsheetId: str = None,
            worksheet: str = '',
            range: str = 'A:Z',
            config: GoogleSheetsConfig = None,
            **kwargs
    ):
        super().__init__()
        self.config = config or GoogleSheetsConfig(
            spreadsheetId=spreadsheetId,
            **kwargs
        )

        self.worksheet = worksheet
        self.range = range

        self.response = None

    @property
    def values(self):
        """row values"""
        if self.response:
            try:
                return self.response['values']
            except Exception as e:
                pass

    def clear(
            self,
            range: str,
            spreadsheetId: str = None,
            **kwargs,
    ):
        """clear rows"""
        try:

            result = self.spreadsheets().values().clear(
                spreadsheetId=spreadsheetId or self.config.spreadsheetId,
                range=range or self.range,
                **kwargs,
            ).execute()

            log.info(f'{result}')
            log.info(f"{result.get('clearedRange')} cells cleared.")
            return result
        except Exception as error:
            log.error(f"An error occurred: {error}")
            return error

    def spreadsheets(self):
        """spreadsheet service"""
        return self.service().spreadsheets()

    def get(
            self,
            spreadsheetId: str = None,
            ranges: str = None,
            includeGridData: bool = False,
            fields: Fields or str = None,
            **kwargs,
    ):
        """get rows"""
        try:
            self.response = self.spreadsheets().get(
                spreadsheetId=spreadsheetId or self.config.spreadsheetId,
                ranges=ranges or self.range,
                includeGridData=includeGridData,
                fields=fields,
                **kwargs,
            ).execute()
            log.info(f'{self.worksheet}!{self.range} ({self.config.spreadsheetId})')
        except Exception as e:
            log.error(f'{e}')

        return self

    def get_values(
            self,
            spreadsheetId: str = None,
            range: str = None,
            **kwargs,
    ):
        """get values"""
        try:
            self.response = self.spreadsheets().values().get(
                spreadsheetId=spreadsheetId or self.config.spreadsheetId,
                range=range or f'{self.worksheet}!{self.range}',
                **kwargs,
            ).execute()

            log.info(str(dict(
                worksheet=self.worksheet,
                range=self.range,
                spreadsheetId=self.config.spreadsheetId,
            )))
        except Exception as e:
            log.error(f'{e}')

        return self

    def list(self):
        # list(pageSize=1).execute()
        log.warning(f'{NotImplemented}')
        return

    def update(
            self,
            spreadsheetId: str = None,
            range: str = None,
            valueInputOption: ValueInputOption = ValueInputOption.USER_ENTERED,
            values: list = None,
    ):
        """update rows"""
        try:

            body = {
                'values': values
            }

            log.debug(f'{body}')

            result = self.spreadsheets().values().update(
                spreadsheetId=spreadsheetId or self.config.spreadsheetId,
                range=range or self.range,
                valueInputOption=valueInputOption,
                body=body
            ).execute()

            log.info(f'{result}')
            log.info(f"{result.get('updatedCells')} cells updated.")
            return result
        except Exception as error:
            log.error(f"An error occurred: {error}")
            return error



from bs4 import BeautifulSoup

from automon.log import logger

from .ssid import Ssid

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class ScanXml:
    def __init__(self, xml: BeautifulSoup = None):
        self._xml = xml

    def __repr__(self):
        return f'{self.summary}'

    def __eq__(self, other):
        if isinstance(other, ScanXml):
            return self._xml == other._xml

    def _bs2dict(self, bs: BeautifulSoup, **kwargs):
        d = {}

        key = None
        value = None

        if 'key' in kwargs:
            key = kwargs['key']

        if 'value' in kwargs:
            value = kwargs['value']

        if hasattr(bs, 'contents'):
            d.update(self._bs2dict(bs.contents, key=key, value=bs.contents))

        elif isinstance(bs, list):
            for tag in bs:
                if tag.name == 'key':
                    key = tag.text
                d.update(self._bs2dict(tag, key=key, value=tag))

        elif key and value:
            d.update({key: value})

        else:
            if '_missed' in d.keys():
                d['_missed'].append({key: value})
            else:
                d['_missed'] = [{key: value}]

        return d

    def byAge(self):
        return sorted(self.ssids, key=(lambda x: x.AGE))

    def byDistance(self):
        return sorted(self.ssids, key=(lambda x: x.RSSI), reverse=True)

    def bySsid(self):
        return sorted(self.ssids, key=(lambda x: x.SSID))

    def load_xml(self, xml: BeautifulSoup):
        self._xml = xml

    @property
    def ssids(self) -> [Ssid]:
        """list of Ssid sorted by closest distance"""

        xml = self._xml
        scan = None
        ssids = None

        try:
            bssids = xml.contents[1].contents[0].contents
            scan = [self._bs2dict(x) for x in bssids]
        except:
            log.error(f'No BSSIDs')

        if scan:
            ssids = [Ssid(ssid) for ssid in scan]
            ssids = sorted(ssids, reverse=True)

        return ssids

    @property
    def summary(self) -> dict:
        summary = {}

        if not self.ssids:
            return f'no ssids'

        ssids = self.ssids
        summary['Total SSID'] = len(ssids)

        if len(ssids) < 0:
            return summary

        summary['SSID'] = {}

        for ssid in ssids:
            count = summary['SSID'].get(ssid.SSID, 0) + 1
            summary['SSID'][ssid.SSID] = count
        summary['SSID'] = {k: v for k, v in sorted(summary['SSID'].items())}

        log.info(f'Total SSID: {summary["Total SSID"]}')
        return summary



class Urls(object):

    def __repr__(self):
        return f'Instagram URLs'

    @property
    def domain(self):
        return 'https://www.instagram.com'

    @property
    def login_page(self):
        return f'{self.domain}/accounts/login/?source=auth_switcher'

    def followers(self, account: str):
        return f'{self.domain}/{account}/followers/'

    def following(self, account: str):
        return f'{self.domain}/{account}/following/'



import unittest

from automon.helpers.subprocessWrapper import Run

run = Run()


class TestRun(unittest.TestCase):

    def test_su(self):
        cmd = [
            'ls',
            '-lh'
        ]
        self.assertTrue(run.run(cmd, sanitize_command=False))


if __name__ == '__main__':
    unittest.main()



import neo4j
import asyncio

from neo4j import GraphDatabase
from queue import Queue

from automon.log import logger
from automon.integrations.neo4jWrapper.cypher import Cypher

from .config import Neo4jConfig
from .results import Results

logger.logging.getLogger('neo4j').setLevel(logger.ERROR)
log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class Neo4jAsyncClient:

    def __init__(self,
                 user: str = None,
                 password: str = None,
                 hosts: list or str = None,
                 config: Neo4jConfig = None) -> neo4j:
        """Neo4j async client

        :param config: Neo4jConfig
        :param user: str
        :param password: str
        :param hosts: list
        :param encrypted: bool
        """

        self.config = config or Neo4jConfig(
            user=user,
            password=password,
            hosts=hosts
        )

        self.client = self._client()
        self.driver = self.client
        self.session = self.client.session()
        self.connected = self.isConnected()
        self.cypher = Queue()

    def __repr__(self):
        return f'{self.__dict__}'

    def _check_connection(self):
        if self.client:
            return True
        return False

    def _client(self):
        try:
            client = GraphDatabase.driver(
                uri=self.config.NEO4J_HOST,
                auth=(self.config.NEO4J_USER, self.config.NEO4J_PASSWORD))
            log.info(f'Connected to neo4j server: {self.config.NEO4J_HOST}')
            return client

        except Exception as e:
            log.error(f'Cannot connect to neo4j server: {self.config.NEO4J_HOST}, {e}')

        return False

    def _consumer(self):
        return

    def _producer(self):
        return

    def isConnected(self):
        return self._check_connection()

    def run(self):
        """Send cyphers in queue"""
        try:
            while not self.cypher.empty():
                cypher = self.cypher.get_nowait()
                log.debug(f'cypher: {cypher}')
                self.session.run(cypher)
            return True
        except Exception as e:
            log.error(f'{e}')

        return False



import unittest

from automon.integrations.seleniumWrapper.browser import SeleniumBrowser
from automon.integrations.seleniumWrapper.config_webdriver_chrome import ChromeWrapper

browser = SeleniumBrowser()
browser.config.webdriver_wrapper = ChromeWrapper()
browser.config.webdriver_wrapper.enable_defaults().enable_headless()


class SeleniumClientTest(unittest.TestCase):
    if browser.run():
        browser.set_window_size(device_type='web-large')

        def test(self):
            while True:

                try:
                    if browser.get('http://bing.com'):
                        self.assertTrue(browser.save_screenshot())
                        self.assertTrue(browser.save_screenshot())
                        self.assertTrue(browser.save_screenshot(folder='./'))

                    browser.quit()
                    break

                except:
                    pass


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.seleniumWrapper.browser import SeleniumBrowser
from automon.integrations.seleniumWrapper.config_webdriver_chrome import ChromeWrapper

browser = SeleniumBrowser()
browser.config.webdriver_wrapper = ChromeWrapper()
browser.config.webdriver_wrapper.enable_defaults().enable_headless()


class SeleniumClientTest(unittest.TestCase):
    if browser.run():
        def test(self):
            browser.quit()


if __name__ == '__main__':
    unittest.main()



from automon.log import logger

from automon.helpers.osWrapper.environ import environ

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class InstagramConfig(object):

    def __init__(self, login: str = None, password: str = None):
        self._login = login or environ('INSTAGRAM_LOGIN', '')
        self._password = password or environ('INSTAGRAM_PASSWORD', '')

    def __repr__(self):
        return str(dict(
            login=self.login,
            password="*" * len(self.password) if self.password else ''
        ))

    @property
    def login(self):
        if not self._login:
            log.error(f'INSTAGRAM_LOGIN')
        return self._login

    @property
    def password(self):
        if not self._password:
            log.error(f'INSTAGRAM_PASSWORD')
        return self._password

    @property
    def is_configured(self):
        if self.login and self.password:
            log.info(f'{True}')
            return True
        log.error(f'{False}')
        return False



import json
import os.path
import datetime

from io import StringIO, BytesIO

from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError

from automon.log import Logging
from automon.helpers import environ

log = Logging(name='GooglePeopleConfig', level=Logging.DEBUG)


class GooglePeopleConfig:

    def __init__(self,
                 token=None,
                 refresh_token=None,
                 id_token=None,
                 token_uri=None,
                 client_id=None,
                 client_secret=None,
                 scopes=None,
                 default_scopes=None,
                 quota_project_id=None,
                 expiry=None,
                 rapt_token=None,
                 refresh_handler=None,
                 enable_reauth_refresh=False,
                 auth_uri=None,
                 auth_provider_x509_cert_url=None,
                 redirect_uris=None,
                 client_type=None
                 ):
        """Google People API config"""

        self._token = token
        self._refresh_token = refresh_token
        self._id_token = id_token
        self._token_uri = token_uri
        self._client_id = client_id
        self._client_secret = client_secret
        self._scopes = scopes
        self._default_scopes = default_scopes
        self._quota_project_id = quota_project_id
        self._expiry = expiry
        self._rapt_token = rapt_token
        self._refresh_handler = refresh_handler
        self._enable_reauth_refresh = enable_reauth_refresh
        self._redirect_uris = redirect_uris

        self._auth_uri = auth_uri
        self._auth_provider_x509_cert_url = auth_provider_x509_cert_url

        self._client_type = client_type

    def __repr__(self):
        return f'{self.__dict__}'

    @property
    def auth_uri(self) -> str:
        return self._auth_uri or environ(
            'GOOGLE_AUTH_URI',
            'https://accounts.google.com/o/oauth2/auth'
        )

    @property
    def auth_provider_x509_cert_url(self) -> str:
        return self._auth_provider_x509_cert_url or environ(
            'GOOGLE_CERT_URL',
            'https://www.googleapis.com/oauth2/v1/certs'
        )

    @property
    def client_id(self) -> str:
        return self._client_id or environ('GOOGLE_CLIENT_ID')

    @property
    def client_secret(self) -> str:
        return self._client_secret or environ('GOOGLE_CLIENT_SECRET')

    @property
    def client_type(self) -> str:

        if self._client_type:
            return self._client_type

        if environ('GOOGLE_OAUTH_WEB'):
            return 'web'

        if environ('GOOGLE_OAUTH_DESKTOP'):
            return 'installed'

        return self._client_type

    @property
    def default_scopes(self) -> list:
        return self._default_scopes

    @property
    def enable_reauth_refresh(self) -> bool:
        return self._enable_reauth_refresh

    @property
    def expiry(self) -> datetime:
        return self._expiry

    @property
    def Credentials(self) -> Credentials:
        return Credentials(
            token=self.token,
            refresh_token=self.refresh_token,
            id_token=self.id_token,
            token_uri=self.token_uri,
            client_id=self.client_id,
            client_secret=self.client_secret,
            scopes=self.scopes,
            default_scopes=self.default_scopes,
            quota_project_id=self._quota_project_id,
            expiry=self.expiry,
            rapt_token=self.rapt_token,
            refresh_handler=self.refresh_handler,
            enable_reauth_refresh=self.enable_reauth_refresh,
        )

    @property
    def id_token(self) -> str:
        return self._id_token

    @property
    def quota_project_id(self) -> str:
        return self._quota_project_id or environ('GOOGLE_PROJECT_ID')

    @property
    def redirect_uris(self) -> list:
        return self._redirect_uris

    @property
    def refresh_handler(self) -> str:
        return self._refresh_handler

    @property
    def rapt_token(self) -> str:
        return self._rapt_token

    @property
    def refresh_token(self) -> str:
        return self._refresh_token or environ('GOOGLE_REFRESH_TOKEN')

    @property
    def scopes(self) -> list:
        return self._scopes or ['https://www.googleapis.com/auth/contacts.readonly']

    @property
    def token(self) -> str:
        return self._token or environ('GOOGLE_TOKEN')

    @property
    def token_uri(self) -> str:
        return self._token_uri or environ(
            'GOOGLE_TOKEN_URI',
            'https://oauth2.googleapis.com/token'
        )

    def credentials_file(self) -> StringIO:
        return StringIO(f'{self.Credentials.to_json()}')

    def credentials_json(self) -> str:
        return self.Credentials.to_json()

    def credentials_dict(self) -> dict:
        return json.loads(self.Credentials.to_json())

    def oauth_dict(self) -> dict:
        """Oauth credentials"""
        if self.client_type:
            return {
                f'{self.client_type}': dict(
                    client_id=self.client_id,
                    project_id=self.quota_project_id,
                    auth_uri=self.auth_uri,
                    token_uri=self.token_uri,
                    auth_provider_x509_cert_url=self.auth_provider_x509_cert_url,
                    client_secret=self.client_secret,
                    redirect_uris=self.redirect_uris
                )
            }

        log.warning(f'Missing client_type')
        return False

    def from_authorized_user_file(self, file: str) -> Credentials:
        """Load token.json"""
        return self.Credentials.from_authorized_user_file(file, self.scopes)

    def isReady(self):
        if self.client_type:
            return True
        if self.oauth_dict():
            return True

        log.warning(f'config is not ready')
        return False

    def load_oauth(self, oauth: dict) -> Credentials:
        if 'installed' in oauth.keys():
            oauth = oauth['installed']
            self._client_type = 'installed'
            return self.update(oauth)

        if 'web' in oauth.keys():
            oauth = oauth['web']
            self._client_type = 'web'
            return self.update(oauth)

        else:
            log.error(msg=f'Unsupported or not an Oauth token. {oauth}', raise_exception=True)

        return self.Credentials

    def load_oauth_file(self, file: str) -> Credentials:
        """Load Oauth credentials.json"""
        with open(file, 'r') as f:
            creds = dict(json.loads(f.read()))
        return self.load_oauth(creds)

    def load_token(self, token: Credentials) -> Credentials:
        return self.update(token.__dict__)

    def load_token_json(self, token: str) -> Credentials:
        token_dict = json.loads(token)
        return self.update(token_dict)

    def update(self, creds: dict) -> Credentials:
        """Update properties"""
        for k, v in creds.items():
            if k[0] == '_':
                k = k
            else:
                k = f'_{k}'
            self.__dict__[k] = v
        return self.Credentials



from automon.log import logger

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class Ssid:
    def __init__(self, ssid: dict):
        self._ssid = dict(sorted(ssid.items()))

        self.AGE = int(self._ssid.get('AGE', 0))
        self.AP_MODE = self._ssid.get('AP_MODE', None)
        self.BSSID = f"{self._ssid.get('BSSID', None)}".upper()
        self.CHANNEL = self._ssid.get('CHANNEL', None)
        self.HT_SECONDARY_CHAN_OFFSET = self._ssid.get('HT_SECONDARY_CHAN_OFFSET', None)
        self.INFO = self._ssid.get('INFO', None)
        self.IE_KEY_80211D_COUNTRY_CODE = self._ssid.get('IE_KEY_80211D_COUNTRY_CODE', None)
        self.IE_KEY_80211D_MAX_POWER = self._ssid.get('IE_KEY_80211D_MAX_POWER', None)
        self.IE_KEY_80211D_NUM_CHANNELS = self._ssid.get('IE_KEY_80211D_NUM_CHANNELS', None)
        self.IE_KEY_WPS_SC_STATE = self._ssid.get('IE_KEY_WPS_SC_STATE', None)
        self.NOISE = int(self._ssid.get('NOISE', 0))
        self.RATES = self._ssid.get('RATES', None)
        self.RSSI = int(self._ssid.get('RSSI', 0))
        self.SSID_B64 = self._ssid.get('SSID', None)
        self.SSID_STR = self._ssid.get('SSID_STR', None)

        # custom
        self.CHANNEL_OFFSET = self.HT_SECONDARY_CHAN_OFFSET
        self.CHANNEL_OFFSET_FULL = f'{self.CHANNEL},+{self.CHANNEL_OFFSET}'
        self.CHANNELS = self.IE_KEY_80211D_NUM_CHANNELS
        self.COUNTRY = self.IE_KEY_80211D_COUNTRY_CODE
        self.DEVICE_INFO = self.INFO
        self.DISTANCE = self.RSSI
        self.MAC = self.BSSID
        self.MAX_POWER = self.IE_KEY_80211D_MAX_POWER
        self.POWER = self.RSSI
        self.SSID = self.SSID_STR
        self.WPS_STATE = self.IE_KEY_WPS_SC_STATE

        self.summary

    def __repr__(self):
        return f'{self.summary}'

    def __eq__(self, other):
        if isinstance(other, Ssid):
            return self.BSSID == other.BSSID

    def __lt__(self, other):
        if isinstance(other, Ssid):
            return self.RSSI < other.RSSI

    @property
    def summary(self):
        summary = f'[rssi: {self.DISTANCE} dBm] ' \
                  f'{self.SSID} ' \
                  f'[ch: {self.CHANNEL}] ' \
                  f'[bssid: {self.MAC}] ' \
                  f'[noise: {self.NOISE}] ' \
                  f'[age: {self.AGE}] '

        log.debug(f'{summary}')
        return summary



import requests

from automon.log import logger

log = logger.logging.getLogger(__name__)
log.setLevel(logger.INFO)


class RequestsConfig(object):

    def __init__(self):
        pass

    def is_ready(self):
        return f'{NotImplemented}'

    def __repr__(self):
        return f'{NotImplemented}'



import unittest

from automon.integrations.requestsWrapper.rest import BaseRestClient


class Test(BaseRestClient):

    def __init__(self):
        BaseRestClient.__init__(self)
        pass


class Client(unittest.TestCase):
    def test_get(self):
        Test().get(url='https://1.1.1.1')


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.vds.client import VdsRestClient
from automon.integrations.vds.config import VdsConfig


class VdsTest(unittest.TestCase):
    def test_VdsConfig(self):
        self.assertTrue(VdsRestClient())

    def test_VdsClient(self):
        self.assertTrue(VdsConfig())


if __name__ == '__main__':
    unittest.main()



import pandas

from automon.log import Logging


def DataFrame(*args, **kwargs) -> pandas.DataFrame:
    log = Logging('DataFrame', level=Logging.ERROR)
    df = pandas.DataFrame(*args, **kwargs)
    log.debug(df)
    return df



from automon.log import logger

from .datatypes import AbstractDataType

log = logger.logging.getLogger(__name__)
log.setLevel(logger.CRITICAL)


class ActionRun(AbstractDataType):
    container: int = None
    id: int = None

    def __repr__(self):
        if self.id:
            return f'{self.id}'
        return f'{self.to_dict()}'



import unittest

from automon.integrations.seleniumWrapper.browser import SeleniumBrowser
from automon.integrations.seleniumWrapper.config_webdriver_chrome import ChromeWrapper

browser = SeleniumBrowser()
browser.config.webdriver_wrapper = ChromeWrapper()
browser.config.webdriver_wrapper.enable_defaults().enable_headless()

agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:112.0) Gecko/20100101 Firefox/112.0'

browser.config.webdriver_wrapper.set_user_agent(agent)


class SeleniumClientTest(unittest.TestCase):
    if browser.run():
        def test_user_agent(self):
            self.assertEqual(browser.user_agent, agent)

            browser.quit()


if __name__ == '__main__':
    unittest.main()



import os
import slack

from automon.log import logger

from .config import SlackConfig
from .bots import BotInfo
from .error import SlackError

log = logger.logging.getLogger(__name__)
log.setLevel(logger.ERROR)


class SlackClient(SlackConfig):

    def __init__(self, token: str = None,
                 config: SlackConfig = None,
                 username: str = None,
                 channel: str = None,
                 icon_emoji: str = None,
                 icon_url: str = None):
        """Slack client
        """

        self.config = config or SlackConfig(token=token, username=username, channel=channel)
        self.client = slack.WebClient(token=self.config.token)

        try:
            self.connected = True if self.client.auth_test() else False
        except:
            self.connected = False

        # TODO: use token to get bot info
        self.username = self.config.username or self._get_bot_info() or ''
        self.channel = self.config.SLACK_DEFAULT_CHANNEL
        self.icon_emoji = icon_emoji or ''
        self.icon_url = icon_url or ''

        self.queue = None

    def _get_bot_info(self):
        if not self.connected:
            return False

        try:
            name = BotInfo(self.client.bots_info()).name
            log.debug(f'Bot name: {name}')
            return name
        except Exception as e:
            error = SlackError(e)
            log.error(
                f'''[{self._get_bot_info.__name__}]\tCouldn't get bot name, missing permission: {error.needed}''')
            return ''

        return ''

    def chat_postMessage(self, channel: str, text: str) -> slack.WebClient.chat_postMessage:

        if self.connected:
            return False

        if not text:
            return SyntaxError

        msg = f'{channel} @{self.username}: {text}'
        log.debug(msg)

        try:
            response = self.client.chat_postMessage(
                text=text, channel=channel, username=self.username,
                icon_emoji=self.icon_emoji, icon_url=self.icon_url)
            assert response["ok"]
            return response
        except Exception as e:
            log.error(e)

        return False

    def files_upload(self, file, filename=None):
        """
        Upload file to slack

        note: username does nothing
        note: icon_emoji does nothing
        note: title takes precedent over filename
        note: title will display full string
        note: filename displays only filename, no extension

        :param file: path to file
        :param filename: (Optional) filename
        :return: Slack response or False
        """

        if not self.connected:
            return False

        # check if file exists
        if not os.path.isfile(file):
            log.error(f'File not found: {file}')
            log.error(f'Working dir: {os.getcwd()}')
            return False

        # get filename
        if not filename:
            filename = os.path.split(file)

        # get file size
        file_size = os.path.getsize(file)

        title = f'{filename} ({file_size}B'

        # TODO: add string validation for `filename`
        # TODO: find a way to change username for files_upload

        response = self.client.files_upload(
            file=file, filename=filename, title=title, username=self.username, channels=self.channel)

        assert response["ok"]
        log.debug(f'File uploaded: {file} ({file_size}B) ({self.username}')

        return response



import unittest

import pandas as pd

from automon.integrations.google.sheets import GoogleSheetsClient

SHEET_NAME = 'Copy of Automated Count DO NOT EDIT'


class MyTestCase(unittest.TestCase):
    def test_authenticate(self):
        sheets_client = GoogleSheetsClient(
            worksheet=SHEET_NAME,
        )

        if not sheets_client.authenticate():
            return

        sheets_client.get_values(
            range=f'{SHEET_NAME}!A:Z'
        )

        sheet_values = sheets_client.values
        sheet_columns = sheet_values[0]
        sheet_data = sheet_values[1:]

        df = pd.DataFrame(data=sheet_data, columns=sheet_columns)
        df = df.dropna(subset=['url'])
        # set df index to match google sheet index numbering
        df.index = df.index + 2

        sheets_client.clear(
            range=f'{SHEET_NAME}!8:5',
        )

        pass


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.google.people import GooglePeopleClient
from automon.integrations.neo4jWrapper import Neo4jClient

c = GooglePeopleClient()
n = Neo4jClient()


class TestClient(unittest.TestCase):

    def test_create_nodes(self):
        if c.isConnected():
            contacts = c.list_connections().contacts
            for contact in contacts:
                n.merge_dict(contact)
            pass


if __name__ == '__main__':
    unittest.main()



import selenium.webdriver

from automon.log import logger
from automon.helpers.osWrapper import environ

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class SeleniumConfig(object):
    def __init__(self):
        self._webdriver = None
        self.webdriver_wrapper = None

        self._cookies_base64 = environ('SELENIUM_COOKIES_BASE64')
        self._cookies_file = environ('SELENIUM_COOKIES_FILE')

    @property
    def webdriver(self):
        try:
            return self.webdriver_wrapper.webdriver
        except:
            return self._webdriver

    @property
    def window_size(self):
        """get window size

        """
        if self.webdriver_wrapper:
            return self.webdriver_wrapper.window_size

    @property
    def cookies_base64(self):
        log.debug(f'{len(self._cookies_base64) if self._cookies_base64 else None}')
        return self._cookies_base64

    @property
    def cookies_file(self):
        log.info(f'{self._cookies_file}')
        return self._cookies_file

    def run(self):
        """run webdriver"""
        run = self.webdriver_wrapper.run()
        self._webdriver = self.webdriver_wrapper.webdriver
        log.info(str(dict(
            webdriver=self.webdriver
        )))
        return run

    def start(self):
        """alias to run"""
        return self.run()

    def quit(self):
        """quit webdriver"""
        return



import os
import warnings
import selenium
import selenium.webdriver

from automon.log import logger
from automon.helpers.osWrapper.environ import environ

from .config_window_size import set_window_size

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class ChromeWrapper(object):

    def __init__(self):
        self._webdriver = None
        self._chrome_options = selenium.webdriver.ChromeOptions()
        self._chromedriver_path = environ('SELENIUM_CHROMEDRIVER_PATH')
        self._ChromeService = None

        self.update_paths()

        self._window_size = set_window_size()

    def __repr__(self):
        if self._webdriver:
            return str(dict(
                name=self.webdriver.name,
                window_size=self.window_size,
                browserVersion=self.browserVersion,
                chromedriverVersion=self.chromedriverVersion,
                chromedriver_path=self.chromedriver_path,
                webdriver=self.webdriver,
            ))

        return f'{__class__}'

    @property
    def browserVersion(self):
        if self.webdriver:
            return self.webdriver.capabilities.get('browserVersion')

    @property
    def chrome_options(self):
        return self._chrome_options

    @property
    def chrome_options_arg(self):
        return self.chrome_options.arguments

    @property
    def chromedriver_path(self):
        return self._chromedriver_path

    @property
    def chromedriverVersion(self):
        if self.webdriver:
            return self.webdriver.capabilities.get('chrome').get('chromedriverVersion')

    @property
    def ChromeService(self):
        return self._ChromeService

    @property
    def webdriver(self) -> selenium.webdriver.Chrome:
        return self._webdriver

    @property
    def window_size(self):
        return self._window_size

    def disable_certificate_verification(self):
        log.warning('Certificates are not verified')
        self.chrome_options.add_argument('--ignore-certificate-errors')
        log.debug(str(dict(
            add_argument='--ignore-certificate-errors'
        )))
        return self

    def disable_extensions(self):
        self.chrome_options.add_argument("--disable-extensions")
        log.debug(str(dict(
            add_argument=f'--disable-extensions'
        )))
        return self

    def disable_infobars(self):
        self.chrome_options.add_argument("--disable-infobars")
        log.debug(str(dict(
            add_argument=f'--disable-infobars'
        )))
        return self

    def disable_notifications(self):
        """Pass the argument 1 to allow and 2 to block

        """
        self.chrome_options.add_experimental_option(
            "prefs", {"profile.default_content_setting_values.notifications": 2}
        )

        log.debug(str(dict(
            add_experimental_option=("prefs", {"profile.default_content_setting_values.notifications": 2})
        )))
        return self

    def disable_sandbox(self):
        self.chrome_options.add_argument('--no-sandbox')
        log.debug(str(dict(
            add_argument=f'--no-sandbox'
        )))
        return self

    def disable_shm(self):
        log.warning('Disabled shm will use disk I/O, and will be slow')
        self.chrome_options.add_argument('--disable-dev-shm-usage')
        log.debug(str(dict(
            add_argument=f'--disable-dev-shm-usage'
        )))
        return self

    def enable_bigshm(self):
        log.warning('Big shm not yet implemented')
        return self

    def enable_defaults(self):
        self.enable_maximized()
        return self

    def enable_fullscreen(self):
        self.chrome_options.add_argument("--start-fullscreen")
        log.debug(str(dict(
            add_argument=f'--start-fullscreen'
        )))
        return self

    def enable_headless(self):
        self.chrome_options.add_argument('headless')
        log.debug(str(dict(
            add_argument='headless'
        )))
        return self

    def enable_notifications(self):
        """Pass the argument 1 to allow and 2 to block

        """
        self.chrome_options.add_experimental_option(
            "prefs", {"profile.default_content_setting_values.notifications": 1}
        )
        log.debug(str(dict(
            add_experimental_option=("prefs", {"profile.default_content_setting_values.notifications": 1})
        )))
        return self

    def enable_maximized(self):
        self.chrome_options.add_argument('--start-maximized')
        log.debug(str(dict(
            add_argument='--start-maximized'
        )))
        return self

    def enable_translate(self, native_language: str = 'en'):
        prefs = {
            "translate_whitelists": {"your native language": native_language},
            "translate": {"enabled": "True"}
        }
        self.chrome_options.add_experimental_option(
            name="prefs",
            value=prefs,
        )

        log.debug(str(dict(
            add_experimental_option=dict(
                name="prefs",
                value=prefs,
            )
        )))
        return self

    def close(self):
        """close

        """
        result = self.webdriver.close()
        log.info(f'{result}')
        return result

    def in_docker(self):
        """Chrome best used with docker

        """
        self.enable_defaults()
        self.enable_headless()
        self.disable_sandbox()
        self.disable_infobars()
        self.disable_extensions()
        self.disable_notifications()
        return self

    def in_headless(self):
        """alias to headless sandboxed

        """
        return self.in_headless_sandboxed()

    def in_headless_sandboxed(self):
        """Headless Chrome with sandbox enabled

        """
        log.warning(
            'Docker does not support sandbox option. '
            'Default shm size is 64m, which will cause chrome driver to crash.'
        )

        self.enable_defaults()
        self.enable_headless()
        return self

    def in_headless_sandbox_disabled(self):
        """Headless Chrome with sandbox disabled

        """
        log.warning('Default shm size is 64m, which will cause chrome driver to crash.')

        self.enable_defaults()
        self.enable_headless()
        self.disable_sandbox()
        return self

    def in_headless_sandbox_disabled_certificate_unverified(self):
        """Headless Chrome with sandbox disabled with no certificate verification

        """
        log.warning('Default shm size is 64m, which will cause chrome driver to crash.')

        self.enable_defaults()
        self.enable_headless()
        self.disable_sandbox()
        self.disable_certificate_verification()
        return self

    def in_headless_sandbox_disabled_shm_disabled(self):
        """Headless Chrome with sandbox disabled

        """
        self.enable_defaults()
        self.enable_headless()
        self.disable_sandbox()
        self.disable_shm()
        return self

    def in_headless_sandbox_disabled_bigshm(self):
        """Headless Chrome with sandbox disabled

        """
        log.warning('Larger shm option is not implemented')

        self.enable_defaults()
        self.enable_headless()
        self.enable_bigshm()
        self.disable_sandbox()
        return self

    def in_remote_driver(self, host: str = '127.0.0.1', port: str = '4444', executor_path: str = '/wd/hub'):
        """Remote Selenium

        """
        log.info(
            f'Remote WebDriver Hub URL: http://{host}:{port}{executor_path}/static/resource/hub.html')

        selenium.webdriver.Remote(
            command_executor=f'http://{host}:{port}{executor_path}',
            desired_capabilities=selenium.webdriver.common.desired_capabilities.DesiredCapabilities.CHROME
        )
        return self

    def in_sandbox(self):
        """Chrome with sandbox enabled

        """
        log.warning(
            'Docker does not support sandbox option. '
            'Default shm size is 64m, which will cause chrome driver to crash.'
        )

        self.enable_defaults()
        return self

    def in_sandbox_disabled(self):
        """Chrome with sandbox disabled

        """
        log.warning('Default shm size is 64m, which will cause chrome driver to crash.')

        self.enable_defaults()
        self.disable_sandbox()
        return self

    def run(self) -> selenium.webdriver.Chrome:
        try:
            if self.chromedriver_path:
                self._ChromeService = selenium.webdriver.ChromeService(
                    executable_path=self.chromedriver_path
                )
                log.debug(str(dict(
                    ChromeService=self.ChromeService
                )))

                self._webdriver = selenium.webdriver.Chrome(
                    service=self.ChromeService,
                    options=self.chrome_options
                )
                log.info(f'{self}')

                return self.webdriver

            self._webdriver = selenium.webdriver.Chrome(options=self.chrome_options)
            log.info(f'{self}')

            return self.webdriver
        except Exception as error:
            log.error(f'{error}')

    def set_chromedriver(self, chromedriver_path: str):
        log.debug(f'{chromedriver_path}')
        self._chromedriver_path = chromedriver_path
        self.update_paths()
        return self

    def set_locale(self, locale: str = 'en'):
        self.chrome_options.add_argument(f"--lang={locale}")
        log.debug(str(dict(
            add_argument=f"--lang={locale}"
        )))
        return self

    def set_locale_experimental(self, locale: str = 'en-US'):
        self.chrome_options.add_experimental_option(
            name='prefs',
            value={'intl.accept_languages': locale}
        )

        log.debug(str(dict(
            add_experimental_option=dict(
                name='prefs',
                value={'intl.accept_languages': locale}
            )
        )))
        return self

    def set_user_agent(self, user_agent: str):
        self.chrome_options.add_argument(f"user-agent={user_agent}")
        log.debug(str(dict(
            add_argument=f"user-agent={user_agent}"
        )))
        return self

    def set_window_size(self, *args, **kwargs):
        self._window_size = set_window_size(*args, **kwargs)
        width, height = self.window_size
        self.webdriver.set_window_size(width=width, height=height)
        log.debug(f'{width}, {height}')
        return self

    def start(self):
        """alias to run

        """
        return self.run()

    def stop_client(self):
        """stop client

        """
        result = self.webdriver.stop_client()
        log.info(f'{result}')
        return result

    def update_paths(self):
        if self.chromedriver_path:
            if self.chromedriver_path not in os.getenv('PATH'):
                os.environ['PATH'] = f"{os.getenv('PATH')}:{self._chromedriver_path}"
                log.debug(str(dict(
                    PATH=os.environ['PATH']
                )))

    def quit(self):
        """quit

        """
        result = self.webdriver.quit()
        log.info(f'{result}')
        return result

    def quit_gracefully(self):
        """gracefully quit webdriver

        """
        try:
            self.close()
            self.quit()
            self.stop_client()
        except Exception as error:
            log.error(f'failed to gracefully quit. {error}')
            return False
        return True



import unittest

from automon.integrations.google.auth import GoogleAuthClient


class MyTestCase(unittest.TestCase):
    def test_authenticate(self):
        test = GoogleAuthClient()
        # scopes = ['https://www.googleapis.com/auth/contacts.readonly']
        # client = AuthClient(serviceName='people', scopes=scopes)
        if test.authenticate():
            self.assertTrue(test.authenticate())


if __name__ == '__main__':
    unittest.main()



import functools
import googleapiclient.http
import googleapiclient.discovery
import google.auth.transport.requests

from automon.log import Logging

from .config import GoogleAuthConfig

log = Logging(name='GoogleAuthClient', level=Logging.DEBUG)


class GoogleAuthClient(object):
    """Google Auth client"""

    def __init__(
            self,
            config: GoogleAuthConfig = None,
            serviceName: str = None,
            scopes: list = None,
            version: str = None,
            **kwargs,
    ):

        self.config = config or GoogleAuthConfig(
            serviceName=serviceName,
            scopes=scopes,
            version=version,
            **kwargs
        )

    def __repr__(self):
        return f'{self.__dict__}'

    @classmethod
    def execute(cls, func):
        return func.execute()

    def _is_connected(func):
        @functools.wraps(func)
        def wrapped(self, *args, **kwargs):
            if self.authenticate():
                return func(self, *args, **kwargs)

        return wrapped

    def authenticate(self) -> bool:
        """authenticate with credentials"""

        try:
            return self.authenticate_oauth()
        except:
            pass

        try:
            return self.authenticate_service_account()
        except:
            pass

        return False

    def authenticate_oauth(self) -> bool:
        """authenticate web token"""

        creds = self.config.Credentials
        refresh_token = creds.refresh_token

        if refresh_token:
            try:
                creds.refresh(google.auth.transport.requests.Request())
                log.info(f'token refresh success')
                return True
            except Exception as e:
                log.error(msg=f'token refresh failed: {e}', enable_traceback=False)

        else:
            # TODO: add google flow() authentication here
            log.info(f'flow login success')
            return True

        return False

    def authenticate_service_account(self) -> bool:
        """authenticate service account"""
        if self.config.Credentials:
            return True
        return False

    def is_connected(self) -> bool:
        """Check if authenticated to make requests"""
        return self.authenticate()

    def service(
            self,
            serviceName: str = None,
            version: str = None,
            http=None,
            discoveryServiceUrl=None,
            developerKey=None,
            model=None,
            requestBuilder=None,
            credentials=None,
            cache_discovery=True,
            cache=None,
            client_options=None,
            adc_cert_path=None,
            adc_key_path=None,
            num_retries=1,
            static_discovery=None,
            always_use_jwt_access=False,
            **kwargs
    ) -> googleapiclient.discovery.build:
        return googleapiclient.discovery.build(
            serviceName=serviceName or self.config.serviceName,
            version=version or self.config.version,
            http=http,
            discoveryServiceUrl=discoveryServiceUrl,
            developerKey=developerKey,
            model=model,
            requestBuilder=requestBuilder or googleapiclient.http.HttpRequest,
            credentials=credentials or self.config.Credentials,
            cache_discovery=cache_discovery,
            cache=cache,
            client_options=client_options,
            adc_cert_path=adc_cert_path,
            adc_key_path=adc_key_path,
            num_retries=num_retries,
            static_discovery=static_discovery,
            always_use_jwt_access=always_use_jwt_access,
            **kwargs,
        )



import json
import requests

from automon.log import logger
from .config import RequestsConfig

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class RequestsClient(object):
    def __init__(self, url: str = None, data: dict = None, headers: dict = None,
                 config: RequestsConfig = None):
        """Wrapper for requests library"""

        self.config = config or RequestsConfig()

        self.url = url
        self.data = data
        self.errors = None
        self.headers = headers
        self.results = None
        self.requests = requests

        if url:
            self.url = url
            self.get(url=self.url, data=self.data, headers=self.headers)

    def __repr__(self):
        return f'{self.__dict__}'

    def __len__(self):
        if self.content:
            len(self.content)

    def _log_result(self):
        if self.results.status_code == 200:
            msg = [
                self.results.request.method,
                self.results.url,
                f'{round(len(self.results.content) / 1024, 2)} KB',
                self.results.status_code,
            ]
            msg = ' '.join(msg)
            return log.debug(msg)

        msg = [
            self.results.request.method,
            self.results.url,
            f'{round(len(self.results.content) / 1024, 2)} KB',
            self.results.status_code,
            self.results.content
        ]

        msg = ' '.join(msg)
        return log.error(msg)

    def _params(self, url, data, headers):
        if url is None:
            url = self.url

        if data is None:
            data = self.data

        if headers is None:
            headers = self.headers

        self.url = url
        self.data = data
        self.headers = headers
        return url, data, headers

    @property
    def content(self):
        if self.results:
            return self.results.content

    @property
    def text(self):
        if self.results:
            return self.results.text

    def delete(self,
               url: str = None,
               data: dict = None,
               headers: dict = None, **kwargs) -> bool:
        """requests.delete"""

        url, data, headers = self._params(url, data, headers)

        try:
            self.results = requests.delete(url=url, data=data, headers=headers, **kwargs)
            self._log_result()
            return True
        except Exception as e:
            self.errors = e
            log.error(f'delete failed. {e}')
        return False

    def get(self,
            url: str = None,
            data: dict = None,
            headers: dict = None, **kwargs) -> bool:
        """requests.get"""

        url, data, headers = self._params(url, data, headers)

        try:
            self.results = requests.get(url=url, data=data, headers=headers, **kwargs)

            log.debug(
                f'{self.results.url} '
                f'{round(len(self.results.content) / 1024, 2)} KB '
                f'{self.results.status_code}'
            )

            return True
        except Exception as e:
            self.errors = e
            log.error(f'{e}')
        return False

    def patch(self,
              url: str = None,
              data: dict = None,
              headers: dict = None, **kwargs) -> bool:
        """requests.patch"""

        url, data, headers = self._params(url, data, headers)

        try:
            self.results = requests.patch(url=url, data=data, headers=headers, **kwargs)

            log.debug(
                f'{self.results.url} '
                f'{round(len(self.results.content) / 1024, 2)} KB '
                f'{self.results.status_code}'
            )

            return True
        except Exception as e:
            self.errors = e
            log.error(f'patch failed. {e}')
        return False

    def post(self,
             url: str = None,
             data: dict = None,
             headers: dict = None, **kwargs) -> bool:
        """requests.post"""

        url, data, headers = self._params(url, data, headers)

        try:
            self.results = requests.post(url=url, data=data, headers=headers, **kwargs)

            log.debug(
                f'{self.results.url} '
                f'{round(len(self.results.content) / 1024, 2)} KB '
                f'{self.results.status_code}'
            )

            return True
        except Exception as e:
            self.errors = e
            log.error(f'post failed. {e}')
        return False

    def put(self,
            url: str = None,
            data: dict = None,
            headers: dict = None, **kwargs) -> bool:
        """requests.put"""

        url, data, headers = self._params(url, data, headers)

        try:
            self.results = requests.put(url=url, data=data, headers=headers, **kwargs)

            log.debug(
                f'{self.results.url} '
                f'{round(len(self.results.content) / 1024, 2)} KB '
                f'{self.results.status_code}'
            )

            return True
        except Exception as e:
            self.errors = e
            log.error(f'put failed. {e}')
        return False

    def to_dict(self):
        if self.results is not None:
            return json.loads(self.results.content)


class Requests(RequestsClient):
    pass



import os
import json
import base64

import google.auth.crypt
import google.oauth2.credentials
import google.oauth2.service_account

from google.auth.transport.requests import Request
from google_auth_oauthlib.flow import InstalledAppFlow

from automon.log import Logging
from automon.helpers import environ

log = Logging(name='GoogleAuthConfig', level=Logging.DEBUG)


class GoogleAuthConfig(object):
    """Google Auth config"""

    def __init__(
            self,
            serviceName: str = None,
            scopes: list = None,
            version: str = None,
    ):
        self.serviceName = serviceName or 'servicemanagement'
        self.scopes = scopes or ['https://www.googleapis.com/auth/cloud-platform.read-only']
        self.version = version or 'v1'

    def __repr__(self):
        return f'{self.__dict__}'

    @property
    def Credentials(self):
        """return Google Credentials object"""
        try:
            if self.CredentialsFile():
                return self.CredentialsFile()
        except:
            pass

        try:
            if self.CredentialsInfo():
                return self.CredentialsInfo()
        except:
            pass

        try:
            if self.CredentialsServiceAccountFile():
                return self.CredentialsServiceAccountFile()
        except:
            pass

        try:
            if self.CredentialsServiceAccountInfo():
                return self.CredentialsServiceAccountInfo()
        except:
            pass

        log.error(f'Missing GOOGLE_CREDENTIALS or GOOGLE_CREDENTIALS_BASE64', enable_traceback=False)

    @property
    def _GOOGLE_CREDENTIALS(self):
        """env var GOOGLE_CREDENTIALS"""
        return environ('GOOGLE_CREDENTIALS')

    @property
    def _GOOGLE_CREDENTIALS_BASE64(self):
        """env var GOOGLE_CREDENTIALS_BASE64"""
        return environ('GOOGLE_CREDENTIALS_BASE64')

    def CredentialsFile(self) -> google.oauth2.credentials.Credentials:
        """return Credentials object for web auth from file"""
        if self._GOOGLE_CREDENTIALS:
            if os.path.exists(self._GOOGLE_CREDENTIALS):
                return google.oauth2.credentials.Credentials.from_authorized_user_file(
                    self._GOOGLE_CREDENTIALS
                )

    def CredentialsInfo(self) -> google.oauth2.credentials.Credentials:
        """return Credentials object for web auth from dict"""
        if self._GOOGLE_CREDENTIALS_BASE64:
            return google.oauth2.credentials.Credentials.from_authorized_user_info(
                self.base64_to_dict()
            )

    def CredentialsServiceAccountFile(self) -> google.oauth2.service_account.Credentials:
        """return Credentials object for service account from file"""
        if self._GOOGLE_CREDENTIALS:
            if os.path.exists(self._GOOGLE_CREDENTIALS):
                return google.oauth2.service_account.Credentials.from_service_account_file(
                    self._GOOGLE_CREDENTIALS
                )

    def CredentialsServiceAccountInfo(self) -> google.oauth2.service_account.Credentials:
        """return Credentials object for service account from dict"""
        if self._GOOGLE_CREDENTIALS_BASE64:
            return google.oauth2.service_account.Credentials.from_service_account_info(
                self.base64_to_dict()
            )

    def base64_to_dict(self, base64_str: str = None) -> dict:
        """convert credential json to dict"""
        if not base64_str and not self._GOOGLE_CREDENTIALS_BASE64:
            raise Exception(f'Missing GOOGLE_CREDENTIALS_BASE6')

        base64_str = base64_str or self._GOOGLE_CREDENTIALS_BASE64
        return json.loads(
            base64.b64decode(base64_str)
        )

    def file_to_base64(self, path: str = None):
        """convert file to base64"""
        if not path and self._GOOGLE_CREDENTIALS:
            path = self._GOOGLE_CREDENTIALS

        with open(path, 'rb') as f:
            return base64.b64encode(f.read()).decode()

    def is_ready(self):
        """return True if configured"""
        if self.Credentials:
            return True



import os

from automon.log import logger
from automon.helpers.sanitation import Sanitation
from automon.helpers.osWrapper.environ import environ

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class Neo4jConfig:
    def __init__(
            self, user: str = None,
            password: str = None,
            hosts: str = None,
            encrypted: bool = None,
            trust: bool = None):
        """Neo4j config
        """

        self.NEO4J_USER = user or environ('NEO4J_USER', '')
        self.NEO4J_PASSWORD = password or environ('NEO4J_PASSWORD', '')
        self.NEO4J_HOST = hosts or environ('NEO4J_HOST', '')

        self.encrypted = encrypted
        self.trust = trust

        if not self.NEO4J_USER: log.error(f'missing NEO4J_USER')
        if not self.NEO4J_PASSWORD: log.error(f'missing NEO4J_PASSWORD')
        if not self.NEO4J_HOST: log.error(f'missing NEO4J_HOST')

    @property
    def is_ready(self) -> bool:
        if self.NEO4J_USER and self.NEO4J_PASSWORD and self.NEO4J_HOST:
            return True
        return False

    def __repr__(self):
        return f'{self.__dict__}'



from slack.web.slack_response import SlackResponse


class BotInfo(SlackResponse):
    def __init__(self, response: dict):
        """{
            "ok": true,
            "bot": {
                "id": "B061F7JD2",
                "deleted": false,
                "name": "beforebot",
                "updated": 1449272004,
                "app_id": "A161CLERW",
                "user_id": "U012ABCDEF",
                "icons": {
                    "image_36": "https://...",
                    "image_48": "https://...",
                    "image_72": "https://..."
                }
            }
        }
        """

        self.status = response.get('ok')
        self.bot = dict(response.get('bot'))
        self.id = self.bot.get('id')
        self.deleted = self.bot.get('deleted')
        self.name = self.bot.get('name')
        self.updated = self.bot.get('updated')
        self.app_id = self.bot.get('app_id')
        self.user_id = self.bot.get('user_id')
        self.icons = self.bot.get('icons')

    def __repr__(self):
        return f'{self.__dict__}'

    def __str__(self):
        return f'{self.__dict__}'

    def __eq__(self, other):
        if not isinstance(other, BotInfo):
            return NotImplemented

        return self.__dict__ == other.__dict__



import math


def human_readable_size(size_bytes):
    if size_bytes == 0:
        return '0B'
    size_name = ('B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB')
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)
    return f'{s} {size_name[i]}'



import unittest

from automon.integrations.seleniumWrapper.user_agents import SeleniumUserAgentBuilder


class MyTestCase(unittest.TestCase):
    def test_filter(self):
        test = SeleniumUserAgentBuilder()
        self.assertTrue(test.filter_agent('applewebkit'))
        self.assertTrue(test.filter_agent('AppleWebKit', case_sensitive=True))

        self.assertFalse(test.filter_agent('xxxxx'))
        self.assertFalse(test.filter_agent('xxxxx', case_sensitive=True))


    def test_random(self):
        test = SeleniumUserAgentBuilder()
        self.assertTrue(test.get_random_agent('applewebkit'))
        self.assertTrue(test.get_random_agent('AppleWebKit', case_sensitive=True))

        self.assertFalse(test.get_random_agent('xxxxx'))
        self.assertFalse(test.get_random_agent('xxxxx', case_sensitive=True))


if __name__ == '__main__':
    unittest.main()



import os

from automon.log import Logging


class SwiftConfig:
    OPENSTACK_USERNAME = os.getenv('OPENSTACK_USERNAME') or ''
    OPENSTACK_PASSWORD = os.getenv('OPENSTACK_PASSWORD') or ''
    OPENSTACK_AUTH_URL = os.getenv('OPENSTACK_AUTH_URL') or ''
    OPENSTACK_PROJECT_ID = os.getenv('OPENSTACK_PROJECT_ID') or ''
    OPENSTACK_PROJECT_NAME = os.getenv('OPENSTACK_PROJECT_NAME') or ''
    OPENSTACK_USER_DOMAIN_NAME = os.getenv('OPENSTACK_USER_DOMAIN_NAME') or ''
    OPENSTACK_PROJECT_DOMAIN_ID = os.getenv('OPENSTACK_PROJECT_DOMAIN_ID') or ''
    OPENSTACK_REGION_NAME = os.getenv('OPENSTACK_REGION_NAME') or 'RegionOne'
    OPENSTACK_INTERFACE = os.getenv('OPENSTACK_INTERFACE') or 'public'
    OPENSTACK_IDENTITY_API_VERSION = os.getenv('OPENSTACK_IDENTITY_API_VERSION') or '3'
    SWIFTCLIENT_INSECURE = os.getenv('SWIFTCLIENT_INSECURE') or 'True'

    def __init__(self):
        self.log = Logging(name=SwiftConfig.__name__, level=Logging.DEBUG)

        if not self.OPENSTACK_USERNAME:
            self.log.warning(f'missing OPENSTACK_USERNAME')
        if not self.OPENSTACK_PASSWORD:
            self.log.warning(f'missing OPENSTACK_PASSWORD')
        if not self.OPENSTACK_AUTH_URL:
            self.log.warning(f'missing OPENSTACK_AUTH_URL')
        if not self.OPENSTACK_PROJECT_ID:
            self.log.warning(f'missing OPENSTACK_PROJECT_ID')
        if not self.OPENSTACK_PROJECT_NAME:
            self.log.warning(f'missing OPENSTACK_PROJECT_NAME')
        if not self.OPENSTACK_USER_DOMAIN_NAME:
            self.log.warning(f'missing OPENSTACK_USER_DOMAIN_NAME')
        if not self.OPENSTACK_PROJECT_DOMAIN_ID:
            self.log.warning(f'missing OPENSTACK_PROJECT_DOMAIN_ID')
        if not self.OPENSTACK_REGION_NAME:
            self.log.warning(f'missing OPENSTACK_REGION_NAME')
        if not self.OPENSTACK_INTERFACE:
            self.log.warning(f'missing OPENSTACK_INTERFACE')
        if not self.OPENSTACK_IDENTITY_API_VERSION:
            self.log.warning(f'missing OPENSTACK_IDENTITY_API_VERSION')
        if not self.SWIFTCLIENT_INSECURE:
            self.log.warning(f'missing SWIFTCLIENT_INSECURE')

    def __eq__(self, other):
        if not isinstance(other, SwiftConfig):
            self.log.warning(f'Not implemented')
            return NotImplemented

        return self.OPENSTACK_USERNAME == other.OPENSTACK_USERNAME and \
               self.OPENSTACK_PASSWORD == other.OPENSTACK_PASSWORD and \
               self.OPENSTACK_AUTH_URL == other.OPENSTACK_AUTH_URL and \
               self.OPENSTACK_PROJECT_ID == other.OPENSTACK_PROJECT_ID and \
               self.OPENSTACK_PROJECT_NAME == other.OPENSTACK_PROJECT_NAME and \
               self.OPENSTACK_PROJECT_DOMAIN_ID == other.OPENSTACK_PROJECT_DOMAIN_ID



import random
import datetime
import statistics

from automon.log import logger
from automon.helpers.sleeper import Sleeper
from automon.integrations.seleniumWrapper import SeleniumBrowser
from automon.integrations.seleniumWrapper.config_webdriver_chrome import ChromeWrapper

log = logger.logging.getLogger(__name__)
log.setLevel(logger.DEBUG)


class FacebookGroups(object):
    _xpath_about = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[3]/div/div/div/div/div/div/div[1]/div/div/div/div/div[2]/a[1]/div[1]/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[3]/div/div/div/div/div/div/div[1]/div/div/div/div/div[1]/div[1]/span',
    ]
    _xpath_popup_close = [
        '/html/body/div[1]/div/div[1]/div/div[5]/div/div/div[1]/div/div[2]/div/div/div/div[1]/div/i',
    ]
    _xpath_content_unavailble = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div/div/div[1]/div[2]/div[1]/span',
    ]
    _xpath_creation_date = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div/div[2]/div/div/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[3]/div/div/div[2]/div/div/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[3]/div/div/div[2]/div/div/span',
    ]
    _xpath_history = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[4]/div/div/div[2]/div/div[2]/span/span',
    ]
    _xpath_title = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[1]/div[2]/div/div/div/div/div[1]/div/div/div/div/div/div[1]/h1/span/a',
    ]
    _xpath_temporarily_blocked = [
        '/html/body/div[1]/div[2]/div[1]/div/div/div[1]/div/div[2]/h2',
        '/html/body/div[1]/div[2]/div[1]/div/div/div[1]/div/div[2]',
    ]
    _xpath_members = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div/div[2]/div/div/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[2]/div/div/div[2]/div/div[1]/span',
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[2]/div/div/div[2]/div/div[1]/span',
    ]
    _xpath_must_login = [
        '/html/body/div[1]/div[1]/div[1]/div/div[2]/div/div',
    ]
    _xpath_posts_today = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[1]/div/div/div[2]/div/div[1]/span',
    ]
    _xpath_posts_monthly = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[1]/div/div/div[2]/div/div[2]/span',
    ]
    _xpath_privacy = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[2]/div/div/div[2]/div/div[1]/span/span',
    ]
    _xpath_privacy_details = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[2]/div/div/div[2]/div/div[2]/span/span',
    ]
    _xpath_visible = [
        '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[3]/div/div/div[2]/div/div[2]/span/span',
    ]

    def __init__(self, url: str = None):
        """Facebook Groups object

        Depends on Selenium"""
        self._url = url

        self._browser = SeleniumBrowser()

        self._rate_per_minute = 2
        self._rate_counter = []
        self._wait_between_retries = random.choice(range(1, 60))

    def content_unavailable(self):
        """This content isn't available right now"""

        try:
            xpath_content_unavailble = self._browser.wait_for_xpath(self._xpath_content_unavailble)
            content_unavailable = self._browser.find_xpath(xpath_content_unavailble).text
            log.debug(content_unavailable)
            return content_unavailable
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            log.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            self.screenshot_error()

    def creation_date(self):

        try:
            xpath_creation_date = self._browser.wait_for_xpath(self._xpath_creation_date)
            creation_date = self._browser.find_xpath(xpath_creation_date).text
            log.debug(creation_date)
            return creation_date
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            log.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            self.screenshot_error()

    def creation_date_timestamp(self):
        if self.creation_date():
            # TODO: convert date to datetime timestamp
            return

    def current_rate_too_fast(self):
        if self.average_rate() == 0 or len(self._rate_counter) < 2:
            log.info(False)
            return False

        if self.average_rate() < self.rate_per_minute():
            log.info(True)
            return True

        return False

    def rate_per_minute(self) -> int:
        rate = int(60 / self._rate_per_minute)
        log.info(str(dict(
            seconds=rate,
        )))
        return rate

    def average_rate(self):
        if self._rate_counter:
            rate = int(statistics.mean(self._rate_counter))
            log.info(str(dict(
                seconds=rate,
            )))
            return rate
        return 0

    def history(self):

        try:
            xpath_history = self._browser.wait_for_xpath(self._xpath_history)
            history = self._browser.find_xpath(xpath_history).text
            log.debug(history)
            return history
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            log.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            self.screenshot_error()

    def temporarily_blocked(self):
        try:
            xpath_temporarily_blocked = self._browser.wait_for_xpath(
                self._xpath_temporarily_blocked
            )
            temporarily_blocked = self._browser.find_xpath(
                xpath_temporarily_blocked
            ).text
            log.debug(temporarily_blocked)
            return temporarily_blocked
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            log.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            self.screenshot_error()

    def members(self):

        try:
            xpath_members = self._browser.wait_for_xpath(self._xpath_members)
            members = self._browser.find_xpath(xpath_members).text
            log.debug(members)
            return members
            # TODO: need to clean up string from members and remove bad chars
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            log.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            self.screenshot_error()

    def members_count(self):

        if self.members():
            count = [x for x in self.members()]
            count = [x for x in count if x in [str(x) for x in range(0, 10)]]
            if count:
                members_count = int(''.join(count)) if count else 0

                log.debug(members_count)
                return members_count

    def must_login(self):
        try:
            xpath_must_login = self._browser.wait_for_xpath(
                self._xpath_must_login
            )
            must_login = self._browser.find_xpath(
                xpath_must_login
            ).text
            log.debug(must_login)
            return must_login
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            log.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            self.screenshot_error()

    def posts_monthly(self):

        try:
            xpath_monthly_posts = self._browser.wait_for_xpath(self._xpath_posts_monthly)
            posts_monthly = self._browser.find_xpath(xpath_monthly_posts).text
            log.debug(posts_monthly)
            return posts_monthly
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            log.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            self.screenshot_error()

    def posts_monthly_count(self):

        if self.posts_monthly():
            count = [x for x in self.posts_monthly()]
            count = [x for x in count if x in [str(x) for x in range(0, 10)]]
            if count:
                posts_monthly_count = int(''.join(count)) if count else 0

                log.debug(posts_monthly_count)
                return posts_monthly_count

    def posts_today(self):

        try:
            xpath_posts_today = self._browser.wait_for_xpath(self._xpath_posts_today)
            posts_today = self._browser.find_xpath(xpath_posts_today).text
            log.debug(posts_today)
            return posts_today
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            log.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            self.screenshot_error()

    def posts_today_count(self):

        if self.posts_today():
            count = [x for x in self.posts_today()]
            count = [x for x in count if x in [str(x) for x in range(0, 10)]]
            if count:
                posts_today_count = int(''.join(count)) if count else 0

                log.debug(posts_today_count)
                return posts_today_count

    def privacy(self):

        try:
            xpath_privacy = self._browser.wait_for_xpath(self._xpath_privacy)
            privacy = self._browser.find_xpath(xpath_privacy).text
            log.debug(privacy)
            return privacy
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            log.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            self.screenshot_error()

    def privacy_details(self):

        try:
            xpath_privacy_details = self._browser.wait_for_xpath(self._xpath_privacy_details)
            privacy_details = self._browser.find_xpath(xpath_privacy_details).text
            log.debug(privacy_details)
            return privacy_details
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            log.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            self.screenshot_error()

    def title(self) -> str:

        try:
            xpath_title = self._browser.wait_for_xpath(self._xpath_title)
            title = self._browser.find_xpath(xpath_title).text
            log.debug(title)
            return title
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            log.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            self.screenshot_error()

    @property
    def url(self) -> str:
        return self.url_cleaner(self._url)

    @staticmethod
    def url_cleaner(url: str):
        """simple url cleaner"""
        if not url:
            return
        if url[-1] == '/':
            url = url[:-1]
        return url

    def visible(self) -> str:

        try:
            xpath_visible = self._browser.wait_for_xpath(self._xpath_visible)
            visible = self._browser.find_xpath(xpath_visible).text
            log.debug(visible)
            return visible
        except Exception as error:
            message, session, stacktrace = self.error_parsing(error)
            log.error(str(dict(
                url=self.url,
                message=message,
                session=session,
                stacktrace=stacktrace,
            )))
            self.screenshot_error()

    @staticmethod
    def error_parsing(error, enabble_stacktrace: bool = False) -> tuple:
        """parses selenium exeption error"""
        error_parsed = f'{error}'.splitlines()
        error_parsed = [f'{x}'.strip() for x in error_parsed]
        message = error_parsed[0]
        session = None
        stacktrace = None
        if len(error_parsed) > 1:
            session = error_parsed[1]
            stacktrace = error_parsed[2:]
            stacktrace = ' '.join(stacktrace)

        if enabble_stacktrace:
            return message, session, stacktrace

        return message, session, 'disabled'

    def get(self, url: str) -> bool:
        """get url"""

        start = datetime.datetime.now().timestamp()

        result = self._browser.get(url=url)
        log.info(str(dict(
            url=url,
            result=result,
        )))
        self.screenshot()

        end = datetime.datetime.now().timestamp()
        seconds_elapsed = int(end - start)

        log.info(str(dict(
            seconds_elapsed=seconds_elapsed,
            result=result,
        )))
        self._rate_counter.append(seconds_elapsed)

        return result

    def get_about(self, rate_limiting: bool = True):
        """get about page"""
        url = f'{self.url}/about'

        if rate_limiting:
            result = self.get_with_rate_limiter(url=url)
        else:
            result = self.get(url=url)

        log.info(str(dict(
            url=url,
            result=result,
        )))
        self.screenshot()
        return result

    def get_with_rate_limiter(
            self,
            url: str,
            retry: int = 0,
            retries: int = 5,
            wait_between_retries: int = None,
            rate_per_minute: int = None,
    ) -> bool:
        """get with rate dynamic limit"""
        if wait_between_retries:
            self._wait_between_retries = wait_between_retries

        result = None
        while retry < retries:

            if self.rate_limited():
                self.rate_limit_increase()

                self._rate_counter.append(self._wait_between_retries)
                Sleeper.seconds(seconds=self._wait_between_retries)
                log.error(str(dict(
                    url=url,
                    retry=retry,
                    retries=retries,
                )))
                continue
            else:
                self.rate_limit_decrease()

            result = self.get(url=url)
            self.screenshot()
            log.info(f'{result}')
            return result

            retry = retry + 1

        log.error(f'{url}')
        self.screenshot_error()
        return result

    def rate_limit_decrease(self, multiplier: int = 0.75):
        before = self._wait_between_retries
        self._wait_between_retries = abs(int(self._wait_between_retries * multiplier))

        if self._wait_between_retries == 0:
            self._wait_between_retries = 1

        log.info(str(dict(
            before=before,
            after=self._wait_between_retries,
            multiplier=multiplier,
        )))
        return self._wait_between_retries

    def rate_limit_increase(self, multiplier: int = 2):
        before = self._wait_between_retries
        self._wait_between_retries = abs(int(self._wait_between_retries * multiplier))

        log.info(str(dict(
            before=before,
            after=self._wait_between_retries,
            multiplier=multiplier,
        )))
        return self._wait_between_retries

    def rate_limited(self):
        """rate limit checker"""
        if self.current_rate_too_fast():
            log.info(True)
            self.screenshot()
            return True

        if self.temporarily_blocked() or self.must_login():
            log.info(True)
            self.screenshot()
            return True

        log.error(False)
        self.screenshot_error()
        return False

    def run(self):
        """run selenium browser"""
        if self._browser:
            log.info(f'{self._browser}')
            return self._browser.run()

    def reset_rate_counter(self):
        self._rate_counter = []
        log.info(self._rate_counter)
        return self._rate_counter

    def restart(self):
        """quit and start new instance of selenium"""
        if self._browser:
            self.quit()
        log.info(f'{self._browser}')
        return self.start()

    def screenshot(self, filename: str = 'screenshot.png'):
        screenshot = self._browser.save_screenshot(filename=filename, folder='.')
        log.info(f'{screenshot}')
        return screenshot

    def screenshot_error(self):
        """get error screenshot"""
        screenshot = self.screenshot(filename='screenshot-error.png')
        log.debug(f'{screenshot}')
        return screenshot

    def screenshot_success(self):
        """get success screenshot"""
        screenshot = self.screenshot(filename='screenshot-success.png')
        log.debug(f'{screenshot}')
        return screenshot

    def set_url(self, url: str) -> str:
        """set new url"""
        self._url = url
        return self.url

    def start(self, headless: bool = True, random_user_agent: bool = False, set_user_agent: str = None):
        """start new instance of selenium"""
        self._browser.config.webdriver_wrapper = ChromeWrapper()

        if headless:
            self._browser.config.webdriver_wrapper.enable_headless().set_locale_experimental()
        else:
            self._browser.config.webdriver_wrapper.set_locale_experimental()

        if random_user_agent:
            self._browser.config.webdriver_wrapper.set_user_agent(
                self._browser.get_random_user_agent()
            )
        elif set_user_agent:
            self._browser.config.webdriver_wrapper.set_user_agent(
                set_user_agent
            )

        log.info(str(dict(
            browser=self._browser
        )))
        browser = self._browser.run()
        self._browser.config.webdriver_wrapper.set_window_size(width=1920 * 0.6, height=1080)
        return browser

    def stop(self):
        """alias to quit"""
        return self.quit()

    def to_dict(self):
        return dict(
            content_unavailable=self.content_unavailable(),
            creation_date=self.creation_date(),
            creation_date_timestamp=self.creation_date_timestamp(),
            history=self.history(),
            members=self.members(),
            members_count=self.members_count(),
            posts_monthly=self.posts_monthly(),
            posts_monthly_count=self.posts_monthly_count(),
            posts_today=self.posts_today(),
            posts_today_count=self.posts_today_count(),
            privacy=self.privacy(),
            privacy_details=self.privacy_details(),
            title=self.title(),
            url=self.url,
            visible=self.visible(),
        )

    def quit(self):
        """quit selenium"""
        if self._browser:
            log.info(f'{self._browser}')
            return self._browser.quit()



import os
import pandas as pd

from queue import Queue
from pandas import DataFrame
from subprocess import Popen, PIPE

from automon.log import Logging


class LdapResult(object):
    def __init__(self, output: list):
        """responses in LDIF format without comments and version

        :param output:
        """
        d = [x.split(': ') for x in output if x]
        columns = [x[0] for x in d]
        data = [[x[1]] for x in d]
        data = dict(zip(columns, data))

        self.df = pd.DataFrame(data=data, columns=columns)

    def __repr__(self) -> str:
        return f'{self.df}'


class LdapClient(object):
    def __init__(self, log_level=Logging.INFO, **kwargs):
        """run ldap commands

        :param log_level:
        :param kwargs:
        """
        self._user = os.getenv('USER')
        self._bash = self._which('bash')
        self.bin = self._which(f'ldapsearch', executable=self._bash)
        self.installed = True if self.bin else False
        # self.connected = self.check_auth()

        self.result = None
        self.results = Queue()

        self._log = Logging(name=LdapClient.__name__, level=log_level, **kwargs)

    def __repr__(self) -> str:
        return f'{self.__dict__}'

    def ldap(self, query: str = None) -> LdapResult:
        """

        :param query:
        :return: LdapResult
        """

        # f'ldapsearch -o ldif-wrap=no -x -h ldap.automon.c -b ou=groups,o=automon -LLL '
        ldap_query = f'ldapsearch -o ldif-wrap=no -x -LLL '
        ldap_query += f'"{query}"' if query else '-z 1 cn=*'

        stdout, stderr = self._run(ldap_query)

        result_ldap = LdapResult(stdout)
        self.result = {'command': ldap_query, 'result': result_ldap}
        self.results.put_nowait(self.result)

        if not result_ldap.df.empty:
            # self._log.debug(f'{result_ldap.df.iloc[0]}')
            self._log.debug(f'FOUND {query} ({self.results.qsize()})')
            # print('o', end='', flush=True)
        else:
            self._log.debug(f'UNKNOWN {query} ({self.results.qsize()})')
            # print('.', end='', flush=True)

        return result_ldap

    def ldap_cn(self, query: str, **kwargs) -> ldap:
        """ldap cn lookup"""
        return self.ldap(query=f'cn={query}', **kwargs)

    def run(self, **kwargs) -> ldap:
        """alias to ldap"""
        return self.ldap(**kwargs)

    def search(self, **kwargs) -> ldap:
        """alias to ldap"""
        return self.ldap(**kwargs)

    @staticmethod
    def to_dict(df: DataFrame) -> dict:
        """dataframe to dict"""
        return df.to_dict()

    def _which(self, cmd: str = None, **kwargs) -> str:
        """run which"""
        cmd = f'which {cmd}'
        out, err = self._run(cmd, shell=True, **kwargs)
        return out[0] if out else False

    @staticmethod
    def _run(command: list or str, shell: bool = True, executable: str = '/bin/bash') -> (list, list):
        """run shell command

        :param command:
        :param shell:
        :param executable:
        :return: Popen stdout, stderr
        """

        # TODO: migrate to Runner
        if shell:
            r = Popen(command, shell=shell, executable=executable,
                      stderr=PIPE, stdout=PIPE, stdin=PIPE).communicate()
        else:
            command = command.split()
            r = Popen(command, shell=shell, executable=executable,
                      stderr=PIPE, stdout=PIPE, stdin=PIPE).communicate()

        out = r[0].decode().splitlines()
        err = r[1].decode().splitlines()

        return out, err
